"""XQTrader CLI - 量化交易引擎命令行工具"""

from __future__ import annotations

import click

from xqtrader import __version__

from cli.commands.backtest import backtest
from cli.commands.strategy import strategy


@click.group()
@click.version_option(version=__version__, prog_name="XQTrader")
def cli() -> None:
    """XQTrader - 量化交易引擎"""
    pass


cli.add_command(backtest)
cli.add_command(strategy)



from __future__ import annotations

import json
import math
from datetime import datetime, timedelta
from pathlib import Path

import pytest

from xqtrader.reports.base import (
    BacktestReport,
    EquityPoint,
    ReportGenerator,
    TradeRecord,
)


def _make_equity_curve(
    start: datetime,
    equities: list[float],
    interval_hours: int = 24,
) -> list[EquityPoint]:
    peak = equities[0]
    points = []
    for i, eq in enumerate(equities):
        ts = start + timedelta(hours=interval_hours * i)
        peak = max(peak, eq)
        dd = peak - eq
        dd_pct = dd / peak if peak > 0 else 0.0
        points.append(EquityPoint(timestamp=ts, equity=eq, drawdown=dd, drawdown_pct=dd_pct))
    return points


def _make_trade(
    trade_id: str,
    entry_time: datetime,
    exit_time: datetime,
    pnl: float,
    entry_price: float = 100.0,
    exit_price: float = 110.0,
    quantity: float = 1.0,
    fees: float = 1.0,
) -> TradeRecord:
    return TradeRecord(
        trade_id=trade_id,
        symbol="BTC/USDT",
        action="LONG",
        entry_time=entry_time,
        entry_price=entry_price,
        exit_time=exit_time,
        exit_price=exit_price,
        quantity=quantity,
        pnl=pnl,
        pnl_pct=pnl / (entry_price * quantity) if entry_price * quantity > 0 else 0.0,
        fees=fees,
        holding_period=exit_time - entry_time,
    )


class TestReportGeneratorGenerate:
    def test_empty_equity_curve(self):
        report = ReportGenerator.generate(
            trade_records=[],
            equity_curve=[],
            strategy_name="test",
            symbol="BTC/USDT",
            interval="1h",
            initial_capital=10000.0,
            final_capital=10000.0,
        )
        assert report.total_trades == 0
        assert report.total_return == 0.0
        assert report.sharpe_ratio == 0.0

    def test_basic_profitable_backtest(self):
        start = datetime(2024, 1, 1)
        equities = [10000, 10200, 10500, 10300, 10800, 11000]
        curve = _make_equity_curve(start, equities)

        trades = [
            _make_trade("t1", start, start + timedelta(days=1), pnl=200),
            _make_trade("t2", start + timedelta(days=1), start + timedelta(days=2), pnl=300),
            _make_trade("t3", start + timedelta(days=2), start + timedelta(days=3), pnl=-200),
            _make_trade("t4", start + timedelta(days=3), start + timedelta(days=4), pnl=500),
            _make_trade("t5", start + timedelta(days=4), start + timedelta(days=5), pnl=200),
        ]

        report = ReportGenerator.generate(
            trade_records=trades,
            equity_curve=curve,
            strategy_name="macd_strategy",
            symbol="BTC/USDT",
            interval="1h",
            initial_capital=10000.0,
            final_capital=11000.0,
        )

        assert report.strategy_name == "macd_strategy"
        assert report.symbol == "BTC/USDT"
        assert report.total_pnl == 1000.0
        assert report.total_return == pytest.approx(0.1)
        assert report.total_trades == 5
        assert report.winning_trades == 4
        assert report.losing_trades == 1
        assert report.win_rate == pytest.approx(0.8)
        assert report.max_drawdown_pct > 0
        assert report.annual_return > 0

    def test_profit_factor(self):
        start = datetime(2024, 1, 1)
        equities = [10000, 10500, 10200]
        curve = _make_equity_curve(start, equities)

        trades = [
            _make_trade("t1", start, start + timedelta(days=1), pnl=500),
            _make_trade("t2", start + timedelta(days=1), start + timedelta(days=2), pnl=-300),
        ]

        report = ReportGenerator.generate(
            trade_records=trades,
            equity_curve=curve,
            strategy_name="test",
            symbol="BTC/USDT",
            interval="1h",
            initial_capital=10000.0,
            final_capital=10200.0,
        )

        assert report.profit_factor == pytest.approx(500 / 300)
        assert report.avg_win == pytest.approx(500.0)
        assert report.avg_loss == pytest.approx(300.0)

    def test_all_winning_trades(self):
        start = datetime(2024, 1, 1)
        equities = [10000, 10500, 11000]
        curve = _make_equity_curve(start, equities)

        trades = [
            _make_trade("t1", start, start + timedelta(days=1), pnl=500),
            _make_trade("t2", start + timedelta(days=1), start + timedelta(days=2), pnl=500),
        ]

        report = ReportGenerator.generate(
            trade_records=trades,
            equity_curve=curve,
            strategy_name="test",
            symbol="BTC/USDT",
            interval="1h",
            initial_capital=10000.0,
            final_capital=11000.0,
        )

        assert report.win_rate == 1.0
        assert report.profit_factor == float("inf")
        assert report.avg_loss == 0.0
        assert report.max_drawdown == 0.0

    def test_max_drawdown_duration(self):
        start = datetime(2024, 1, 1)
        # Peak at day 0, drawdown from day 1-3, recovery at day 4
        equities = [10000, 9500, 9200, 9800, 10000]
        curve = _make_equity_curve(start, equities)

        report = ReportGenerator.generate(
            trade_records=[],
            equity_curve=curve,
            strategy_name="test",
            symbol="BTC/USDT",
            interval="1h",
            initial_capital=10000.0,
            final_capital=10000.0,
        )

        assert report.max_drawdown == pytest.approx(800.0)
        assert report.max_drawdown_pct == pytest.approx(0.08)
        # Drawdown from day 0 (peak) to day 3 (last point before recovery)
        assert report.max_drawdown_duration == timedelta(hours=72)

    def test_open_trades_excluded_from_stats(self):
        start = datetime(2024, 1, 1)
        equities = [10000, 10500]
        curve = _make_equity_curve(start, equities)

        open_trade = TradeRecord(
            trade_id="t1",
            symbol="BTC/USDT",
            action="LONG",
            entry_time=start,
            entry_price=100.0,
            quantity=1.0,
        )

        report = ReportGenerator.generate(
            trade_records=[open_trade],
            equity_curve=curve,
            strategy_name="test",
            symbol="BTC/USDT",
            interval="1h",
            initial_capital=10000.0,
            final_capital=10500.0,
        )

        assert report.total_trades == 0


class TestReportGeneratorExport:
    def test_export_json_string(self):
        start = datetime(2024, 1, 1)
        equities = [10000, 10500]
        curve = _make_equity_curve(start, equities)

        report = ReportGenerator.generate(
            trade_records=[],
            equity_curve=curve,
            strategy_name="test",
            symbol="BTC/USDT",
            interval="1h",
            initial_capital=10000.0,
            final_capital=10500.0,
        )

        result = ReportGenerator.export(report)
        data = json.loads(result)
        assert data["summary"]["strategy_name"] == "test"
        assert data["returns"]["total_pnl"] == 500.0

    def test_export_json_file(self, tmp_path):
        start = datetime(2024, 1, 1)
        equities = [10000, 10500]
        curve = _make_equity_curve(start, equities)

        report = ReportGenerator.generate(
            trade_records=[],
            equity_curve=curve,
            strategy_name="test",
            symbol="BTC/USDT",
            interval="1h",
            initial_capital=10000.0,
            final_capital=10500.0,
        )

        file_path = str(tmp_path / "reports" / "test_report.json")
        ReportGenerator.export(report, format="json", path=file_path)

        assert Path(file_path).exists()
        data = json.loads(Path(file_path).read_text())
        assert data["summary"]["strategy_name"] == "test"

    def test_export_unsupported_format(self):
        start = datetime(2024, 1, 1)
        curve = _make_equity_curve(start, [10000])

        report = ReportGenerator.generate(
            trade_records=[],
            equity_curve=curve,
            strategy_name="test",
            symbol="BTC/USDT",
            interval="1h",
            initial_capital=10000.0,
            final_capital=10000.0,
        )

        with pytest.raises(ValueError, match="Unsupported format"):
            ReportGenerator.export(report, format="html")


class TestDailyAndMonthlyReturns:
    def test_daily_returns_calculation(self):
        start = datetime(2024, 1, 1)
        equities = [10000, 10100, 10300, 10200]
        curve = _make_equity_curve(start, equities)

        report = ReportGenerator.generate(
            trade_records=[],
            equity_curve=curve,
            strategy_name="test",
            symbol="BTC/USDT",
            interval="1h",
            initial_capital=10000.0,
            final_capital=10200.0,
        )

        assert len(report.daily_returns) == 3
        assert report.daily_returns[0] == pytest.approx(0.01)
        assert report.daily_returns[1] == pytest.approx(200 / 10100)

    def test_monthly_returns_populated(self):
        start = datetime(2024, 1, 1)
        # Span 2 months
        equities = [10000]
        for i in range(1, 60):
            equities.append(10000 + i * 10)
        curve = _make_equity_curve(start, equities)

        report = ReportGenerator.generate(
            trade_records=[],
            equity_curve=curve,
            strategy_name="test",
            symbol="BTC/USDT",
            interval="1h",
            initial_capital=10000.0,
            final_capital=equities[-1],
        )

        assert len(report.monthly_returns) > 0


class TestSharpeAndSortino:
    def test_sharpe_positive_returns(self):
        start = datetime(2024, 1, 1)
        equities = [10000 + i * 50 for i in range(30)]
        curve = _make_equity_curve(start, equities)

        report = ReportGenerator.generate(
            trade_records=[],
            equity_curve=curve,
            strategy_name="test",
            symbol="BTC/USDT",
            interval="1h",
            initial_capital=10000.0,
            final_capital=equities[-1],
        )

        assert report.sharpe_ratio > 0

    def test_sortino_no_downside(self):
        start = datetime(2024, 1, 1)
        equities = [10000 + i * 50 for i in range(10)]
        curve = _make_equity_curve(start, equities)

        report = ReportGenerator.generate(
            trade_records=[],
            equity_curve=curve,
            strategy_name="test",
            symbol="BTC/USDT",
            interval="1h",
            initial_capital=10000.0,
            final_capital=equities[-1],
        )

        assert report.sortino_ratio == float("inf")



"""数据中心测试

测试交易所适配器、缓存和熔断器功能。
"""

from __future__ import annotations

import asyncio
import time
import pytest

from xqtrader.data.cache import LRUCache, DataCenterCache
from xqtrader.data.circuit_breaker import CircuitBreaker, CircuitState, CircuitBreakerOpenError
from xqtrader.data.adapters.base import ExchangeAdapter, OHLCVData


class TestLRUCache:
    """LRU 缓存测试"""

    def test_basic_get_set(self):
        cache: LRUCache[str] = LRUCache(max_size=10)
        cache.set("key1", "value1")
        assert cache.get("key1") == "value1"

    def test_cache_miss(self):
        cache: LRUCache[str] = LRUCache(max_size=10)
        assert cache.get("nonexistent") is None

    def test_ttl_expiration(self):
        cache: LRUCache[str] = LRUCache(max_size=10, default_ttl=0.1)
        cache.set("key1", "value1")
        assert cache.get("key1") == "value1"
        time.sleep(0.15)
        assert cache.get("key1") is None

    def test_custom_ttl(self):
        cache: LRUCache[str] = LRUCache(max_size=10, default_ttl=10.0)
        cache.set("key1", "value1", ttl=0.1)
        assert cache.get("key1") == "value1"
        time.sleep(0.15)
        assert cache.get("key1") is None

    def test_lru_eviction(self):
        cache: LRUCache[str] = LRUCache(max_size=3)
        cache.set("key1", "value1")
        cache.set("key2", "value2")
        cache.set("key3", "value3")
        # 访问 key1，使其成为最近使用
        cache.get("key1")
        # 添加新项，key2 应被淘汰
        cache.set("key4", "value4")
        assert cache.get("key2") is None
        assert cache.get("key1") == "value1"
        assert cache.get("key3") == "value3"
        assert cache.get("key4") == "value4"

    def test_hit_rate(self):
        cache: LRUCache[str] = LRUCache(max_size=10)
        cache.set("key1", "value1")
        cache.get("key1")  # hit
        cache.get("key1")  # hit
        cache.get("key2")  # miss
        assert cache.hit_rate == 2 / 3

    def test_clear_expired(self):
        cache: LRUCache[str] = LRUCache(max_size=10, default_ttl=0.1)
        cache.set("key1", "value1")
        cache.set("key2", "value2", ttl=10.0)
        time.sleep(0.15)
        cleared = cache.clear_expired()
        assert cleared == 1
        assert cache.get("key1") is None
        assert cache.get("key2") == "value2"


class TestDataCenterCache:
    """数据中心缓存测试"""

    def test_kline_cache(self):
        cache = DataCenterCache()
        data = {"ohlcv": {"close": [100, 101, 102]}}
        cache.set_kline("BTC/USDT", "1h", data)
        cached = cache.get_kline("BTC/USDT", "1h")
        assert cached == data

    def test_ticker_cache(self):
        cache = DataCenterCache(ticker_ttl=1.0)
        data = {"last_price": 50000.0}
        cache.set_ticker("BTC/USDT", data)
        assert cache.get_ticker("BTC/USDT") == data

    def test_orderbook_cache(self):
        cache = DataCenterCache()
        data = {"bids": [[50000, 1.0]], "asks": [[50001, 1.0]]}
        cache.set_orderbook("BTC/USDT", data)
        assert cache.get_orderbook("BTC/USDT") == data

    def test_stats(self):
        cache = DataCenterCache()
        cache.set_kline("BTC/USDT", "1h", {})
        stats = cache.stats()
        assert "kline" in stats
        assert "ticker" in stats
        assert "orderbook" in stats
        assert stats["kline"]["size"] == 1


class TestCircuitBreaker:
    """熔断器测试"""

    def test_initial_state_closed(self):
        cb = CircuitBreaker(failure_threshold=3)
        assert cb.state == CircuitState.CLOSED
        assert cb.allow_request() is True

    def test_open_after_failures(self):
        cb = CircuitBreaker(failure_threshold=3)
        cb.record_failure()
        cb.record_failure()
        assert cb.state == CircuitState.CLOSED
        cb.record_failure()
        assert cb.state == CircuitState.OPEN
        assert cb.allow_request() is False

    def test_half_open_after_timeout(self):
        cb = CircuitBreaker(failure_threshold=2, timeout=0.1)
        cb.record_failure()
        cb.record_failure()
        assert cb.state == CircuitState.OPEN
        time.sleep(0.15)
        assert cb.state == CircuitState.HALF_OPEN
        assert cb.allow_request() is True

    def test_close_after_success_in_half_open(self):
        cb = CircuitBreaker(failure_threshold=2, success_threshold=2, timeout=0.1)
        cb.record_failure()
        cb.record_failure()
        time.sleep(0.15)
        assert cb.state == CircuitState.HALF_OPEN
        cb.record_success()
        assert cb.state == CircuitState.HALF_OPEN
        cb.record_success()
        assert cb.state == CircuitState.CLOSED

    def test_reopen_on_failure_in_half_open(self):
        cb = CircuitBreaker(failure_threshold=2, timeout=0.1)
        cb.record_failure()
        cb.record_failure()
        time.sleep(0.15)
        assert cb.state == CircuitState.HALF_OPEN
        cb.record_failure()
        assert cb.state == CircuitState.OPEN

    def test_reset(self):
        cb = CircuitBreaker(failure_threshold=2)
        cb.record_failure()
        cb.record_failure()
        assert cb.state == CircuitState.OPEN
        cb.reset()
        assert cb.state == CircuitState.CLOSED
        assert cb.allow_request() is True

    def test_circuit_breaker_open_error(self):
        cb = CircuitBreaker(failure_threshold=1, name="test_breaker")
        cb.record_failure()
        with pytest.raises(CircuitBreakerOpenError) as exc_info:
            if not cb.allow_request():
                raise CircuitBreakerOpenError(cb)
        assert "test_breaker" in str(exc_info.value)


class TestExchangeAdapter:
    """交易所适配器工具方法测试"""

    def test_normalize_symbol(self):
        assert ExchangeAdapter.normalize_symbol("BTCUSDT") == "BTC/USDT"
        assert ExchangeAdapter.normalize_symbol("BTC/USDT") == "BTC/USDT"
        assert ExchangeAdapter.normalize_symbol("ETHBTC") == "ETH/BTC"
        assert ExchangeAdapter.normalize_symbol("SOLUSDC") == "SOL/USDC"

    def test_denormalize_symbol(self):
        assert ExchangeAdapter.denormalize_symbol("BTC/USDT") == "BTCUSDT"
        assert ExchangeAdapter.denormalize_symbol("ETH/BTC") == "ETHBTC"

    def test_normalize_interval(self):
        assert ExchangeAdapter.normalize_interval("1min") == "1m"
        assert ExchangeAdapter.normalize_interval("1hour") == "1h"
        assert ExchangeAdapter.normalize_interval("1day") == "1d"
        assert ExchangeAdapter.normalize_interval("1h") == "1h"


class TestOHLCVData:
    """OHLCV 数据结构测试"""

    def test_empty(self):
        data = OHLCVData.empty()
        assert len(data) == 0
        assert data.timestamps == []

    def test_to_dict(self):
        data = OHLCVData(
            timestamps=[1000, 2000],
            open=[100.0, 101.0],
            high=[105.0, 106.0],
            low=[99.0, 100.0],
            close=[104.0, 105.0],
            volume=[1000.0, 1100.0],
        )
        d = data.to_dict()
        assert d["timestamps"] == [1000, 2000]
        assert d["close"] == [104.0, 105.0]
        assert len(data) == 2


# ==================== 集成测试（需要网络） ====================

@pytest.mark.asyncio
@pytest.mark.skip(reason="需要网络连接，手动运行")
async def test_binance_adapter_fetch_ohlcv():
    """测试 Binance 适配器获取 K 线（需要网络）"""
    from xqtrader.data.adapters.binance import BinanceAdapter
    from xqtrader.data.adapters.base import MarketType

    async with BinanceAdapter(MarketType.SPOT) as adapter:
        ohlcv = await adapter.fetch_ohlcv("BTC/USDT", "1h", limit=10)
        assert len(ohlcv) > 0
        assert ohlcv.close[-1] > 0


@pytest.mark.asyncio
@pytest.mark.skip(reason="需要网络连接，手动运行")
async def test_binance_futures_adapter():
    """测试 Binance 永续合约适配器（需要网络）"""
    from xqtrader.data.adapters.binance import BinanceAdapter
    from xqtrader.data.adapters.base import MarketType

    async with BinanceAdapter(MarketType.FUTURES) as adapter:
        ohlcv = await adapter.fetch_ohlcv("BTC/USDT", "1h", limit=10)
        assert len(ohlcv) > 0

        ticker = await adapter.fetch_ticker("BTC/USDT")
        assert ticker.last_price > 0


@pytest.mark.asyncio
@pytest.mark.skip(reason="需要网络连接，手动运行")
async def test_data_center_service():
    """测试数据中心服务（需要网络）"""
    from xqtrader.data import DataCenterService, MarketDataRequest, MarketType

    async with DataCenterService(market_type=MarketType.SPOT) as dc:
        data = await dc.get_market_data(
            MarketDataRequest(symbol="BTC/USDT", interval="1h", limit=10)
        )
        assert data["metadata"]["count"] > 0
        assert len(data["ohlcv"]["close"]) > 0

        # 测试缓存命中
        data2 = await dc.get_market_data(
            MarketDataRequest(symbol="BTC/USDT", interval="1h", limit=10)
        )
        stats = dc.cache_stats()
        assert stats["kline"]["hits"] >= 1



"""qt strategy - 策略管理命令"""

from __future__ import annotations

from pathlib import Path

import click

from cli.console import console


@click.group()
def strategy() -> None:
    """策略管理"""
    pass


@strategy.command("list")
@click.option(
    "--dir", "-d",
    type=click.Path(exists=True, file_okay=False),
    default=".",
    help="策略目录",
)
def list_strategies(dir: str) -> None:
    """列出目录下的策略文件"""
    from rich.table import Table

    from xqtrader.strategies.base import StrategyLoader

    loader = StrategyLoader(dir)
    skipped = []
    for file_path in Path(dir).glob("*.py"):
        try:
            loader.load_strategy_from_file(str(file_path))
        except Exception:
            skipped.append(file_path.name)
    entries = loader.loaded_strategies

    if skipped:
        console.print(f"[dim]跳过非策略文件: {', '.join(skipped)}[/dim]")

    if not entries:
        console.print(f"[yellow]在 {dir} 下未找到策略文件[/yellow]")
        return

    table = Table(title="可用策略")
    table.add_column("名称", style="cyan")
    table.add_column("版本", style="green")
    table.add_column("描述")
    table.add_column("文件", style="dim")

    for _key, entry in entries.items():
        meta = entry.metadata
        table.add_row(
            meta.get("name", "N/A"),
            meta.get("version", "N/A"),
            meta.get("description", ""),
            str(Path(entry.file_path).name),
        )

    console.print(table)


@strategy.command("validate")
@click.argument("file_path", type=click.Path(exists=True, dir_okay=False))
def validate_strategy(file_path: str) -> None:
    """校验策略文件是否可加载"""
    from xqtrader.strategies.base import StrategyLoader

    loader = StrategyLoader(str(Path(file_path).parent))
    instance = loader.load_strategy_from_file(file_path)

    if instance is None:
        console.print(f"[red]校验失败:[/red] 未找到有效的 BaseStrategy 子类")
        raise SystemExit(1)

    console.print(f"[green]校验通过[/green]")
    console.print(f"  名称: {instance.name}")
    console.print(f"  版本: {instance.version}")
    console.print(f"  描述: {instance.description or '(无)'}")

    # 检查 execute 方法
    from xqtrader.strategies.base import BaseStrategy
    if hasattr(instance, "execute") and callable(instance.execute):
        console.print(f"  execute: [green]已实现[/green]")
    else:
        console.print(f"  execute: [red]未实现[/red]")

    # 检查指标需求
    indicator_reqs = instance.get_indicator_requirements()
    if indicator_reqs:
        console.print(f"  指标需求: {', '.join(indicator_reqs.keys())}")
    else:
        console.print(f"  指标需求: (无)")


@strategy.command("new")
@click.argument("name")
@click.option("--output", "-o", type=click.Path(), default=None, help="输出路径")
def new_strategy(name: str, output: str | None) -> None:
    """生成策略模板文件"""
    template = '''"""策略: {name}"""

from __future__ import annotations

from xqtrader.strategies.base import BaseStrategy, StrategyContext, StrategyResult


class {class_name}(BaseStrategy):
    def __init__(self) -> None:
        super().__init__(
            name="{name}",
            version="1.0.0",
            description="",
        )

    def execute(self, context: StrategyContext) -> StrategyResult:
        closes = context.market_data.get("close", [])
        if not closes:
            return StrategyResult(
                signals=[], indicators={{}}, metadata={{}},
                execution_time=0.0, success=True,
            )

        current_price = closes[-1]

        # TODO: 实现你的策略逻辑
        signals = []

        return StrategyResult(
            signals=signals,
            indicators={{}},
            metadata={{"price": current_price}},
            execution_time=0.0,
            success=True,
        )
'''

    class_name = "".join(word.capitalize() for word in name.replace("-", "_").split("_")) + "Strategy"
    content = template.format(name=name, class_name=class_name)

    if output is None:
        output = f"{name.replace('-', '_')}.py"

    path = Path(output)
    if path.exists():
        console.print(f"[red]文件已存在: {output}[/red]")
        raise SystemExit(1)

    path.write_text(content, encoding="utf-8")
    console.print(f"[green]策略模板已生成: {output}[/green]")



"""熔断器实现

提供服务熔断保护，防止故障扩散。
"""

from __future__ import annotations

import time
from dataclasses import dataclass
from enum import Enum
from typing import Optional


class CircuitState(str, Enum):
    """熔断器状态"""

    CLOSED = "closed"  # 正常状态
    OPEN = "open"  # 熔断状态
    HALF_OPEN = "half_open"  # 半开状态（试探）


@dataclass
class CircuitStats:
    """熔断器统计"""

    state: CircuitState
    failure_count: int
    success_count: int
    last_failure_time: Optional[float]
    last_state_change: float


class CircuitBreaker:
    """熔断器

    状态转换：
    - CLOSED -> OPEN: 失败次数达到阈值
    - OPEN -> HALF_OPEN: 等待超时后自动进入
    - HALF_OPEN -> CLOSED: 试探请求成功
    - HALF_OPEN -> OPEN: 试探请求失败
    """

    def __init__(
        self,
        failure_threshold: int = 5,
        success_threshold: int = 2,
        timeout: float = 30.0,
        name: str = "default",
    ) -> None:
        """
        Args:
            failure_threshold: 触发熔断的连续失败次数
            success_threshold: 半开状态下恢复所需的成功次数
            timeout: 熔断超时时间（秒）
            name: 熔断器名称（用于日志）
        """
        self.failure_threshold = failure_threshold
        self.success_threshold = success_threshold
        self.timeout = timeout
        self.name = name

        self._state = CircuitState.CLOSED
        self._failure_count = 0
        self._success_count = 0
        self._last_failure_time: Optional[float] = None
        self._last_state_change = time.time()

    @property
    def state(self) -> CircuitState:
        """当前状态（自动检查是否应从 OPEN 转为 HALF_OPEN）"""
        if self._state == CircuitState.OPEN:
            if self._should_try_reset():
                self._set_state(CircuitState.HALF_OPEN)
        return self._state

    def _should_try_reset(self) -> bool:
        """检查是否应该尝试重置"""
        if self._last_failure_time is None:
            return True
        return time.time() - self._last_failure_time >= self.timeout

    def _set_state(self, state: CircuitState) -> None:
        """设置状态"""
        if self._state != state:
            self._state = state
            self._last_state_change = time.time()
            if state == CircuitState.HALF_OPEN:
                self._success_count = 0

    def allow_request(self) -> bool:
        """检查是否允许请求"""
        current_state = self.state  # 会自动检查 OPEN -> HALF_OPEN

        if current_state == CircuitState.CLOSED:
            return True
        elif current_state == CircuitState.HALF_OPEN:
            return True  # 允许试探请求
        else:  # OPEN
            return False

    def record_success(self) -> None:
        """记录成功"""
        if self._state == CircuitState.HALF_OPEN:
            self._success_count += 1
            if self._success_count >= self.success_threshold:
                self._set_state(CircuitState.CLOSED)
                self._failure_count = 0
        elif self._state == CircuitState.CLOSED:
            self._failure_count = 0

    def record_failure(self) -> None:
        """记录失败"""
        self._failure_count += 1
        self._last_failure_time = time.time()

        if self._state == CircuitState.HALF_OPEN:
            # 半开状态下失败，立即熔断
            self._set_state(CircuitState.OPEN)
        elif self._state == CircuitState.CLOSED:
            if self._failure_count >= self.failure_threshold:
                self._set_state(CircuitState.OPEN)

    def reset(self) -> None:
        """重置熔断器"""
        self._state = CircuitState.CLOSED
        self._failure_count = 0
        self._success_count = 0
        self._last_failure_time = None
        self._last_state_change = time.time()

    def stats(self) -> CircuitStats:
        """获取熔断器统计"""
        return CircuitStats(
            state=self.state,
            failure_count=self._failure_count,
            success_count=self._success_count,
            last_failure_time=self._last_failure_time,
            last_state_change=self._last_state_change,
        )

    def is_open(self) -> bool:
        """是否处于熔断状态"""
        return self.state == CircuitState.OPEN


class CircuitBreakerOpenError(Exception):
    """熔断器开启时抛出的异常"""

    def __init__(self, breaker: CircuitBreaker):
        self.breaker = breaker
        super().__init__(
            f"Circuit breaker '{breaker.name}' is open. "
            f"Retry after {breaker.timeout} seconds."
        )



"""K 线时间框架重采样器

将低时间框架 K 线聚合到高时间框架，例如：4 根 15m K 线 -> 1 根 1h K 线
基于时间戳边界对齐，确保聚合结果与交易所原生 K 线一致。

设计原则：
1. 时间边界对齐 - 根据时间戳判断 K 线所属周期，而非简单计数
2. 即时输出 - 在目标周期最后一根源 K 线时立即输出，避免延迟
3. 增量计算 - 每根 K 线只处理一次，O(1) 复杂度
"""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Optional

from .incremental import IndicatorBar


@dataclass
class OhlcvResampler:
    """将低时间框架 K 线聚合到高时间框架

    基于时间戳边界对齐，在目标周期最后一根源 K 线时即时输出。

    例如：2 根 30m K 线 → 1 根 1h K 线
    - 00:00 的 30m bar → 开始聚合（不输出）
    - 00:30 的 30m bar → 检测到是周期最后一根，立即输出 00:00 的 1h bar

    Attributes:
        source_tf: 源时间框架，如 "30m"
        target_tf: 目标时间框架，如 "1h"
        ratio: 聚合比例，如 2 (30m * 2 = 1h)
    """

    source_tf: str
    target_tf: str
    ratio: int

    _source_seconds_ms: int = field(default=0, init=False)
    _target_seconds_ms: int = field(default=0, init=False)
    _current_period_start: Optional[int] = field(default=None, init=False)
    _pending_open: Optional[float] = field(default=None, init=False)
    _pending_high: float = field(default=float("-inf"), init=False)
    _pending_low: float = field(default=float("inf"), init=False)
    _pending_close: float = field(default=0.0, init=False)
    _pending_volume: float = field(default=0.0, init=False)
    _pending_ts: int = field(default=0, init=False)
    _count: int = field(default=0, init=False)

    def __post_init__(self) -> None:
        """初始化时间框架的毫秒数"""
        self._source_seconds_ms = _tf_to_seconds(self.source_tf) * 1000
        self._target_seconds_ms = _tf_to_seconds(self.target_tf) * 1000

    def _get_period_start(self, timestamp_ms: int) -> int:
        """计算时间戳所属的目标时间框架周期起始时间

        Args:
            timestamp_ms: 毫秒时间戳

        Returns:
            周期起始时间（毫秒）
        """
        return (timestamp_ms // self._target_seconds_ms) * self._target_seconds_ms

    def _is_period_last_bar(self, bar_timestamp_ms: int) -> bool:
        """判断当前 bar 是否是目标周期的最后一根

        Args:
            bar_timestamp_ms: 当前 bar 的时间戳（毫秒）

        Returns:
            如果是周期最后一根返回 True
        """
        period_start = self._get_period_start(bar_timestamp_ms)
        period_end = period_start + self._target_seconds_ms
        next_bar_ts = bar_timestamp_ms + self._source_seconds_ms
        return next_bar_ts >= period_end

    def add(self, bar: IndicatorBar) -> Optional[IndicatorBar]:
        """添加一根源 K 线，返回聚合后的 K 线（如果完成）

        基于时间戳边界对齐，在目标周期最后一根源 K 线时即时输出。

        Args:
            bar: 源时间框架的 K 线数据

        Returns:
            聚合完成时返回目标时间框架的 K 线，否则返回 None
        """
        bar_period_start = self._get_period_start(bar.timestamp)
        result: Optional[IndicatorBar] = None

        # 检查是否进入新周期（处理数据跳跃的情况）
        if self._current_period_start is not None and bar_period_start != self._current_period_start:
            # 输出上一周期的聚合结果（如果有未输出的数据）
            if self._pending_open is not None:
                result = self._create_output_bar()
                self._reset()

        # 更新当前周期
        self._current_period_start = bar_period_start

        # 聚合当前 bar
        if self._pending_open is None:
            self._pending_open = bar.open
            self._pending_ts = bar_period_start  # 使用周期起始时间作为时间戳

        self._pending_high = max(self._pending_high, bar.high)
        self._pending_low = min(self._pending_low, bar.low)
        self._pending_close = bar.close
        self._pending_volume += bar.volume
        self._count += 1

        # 检查是否是目标周期的最后一根 bar，如果是则立即输出
        if self._is_period_last_bar(bar.timestamp):
            result = self._create_output_bar()
            self._reset()

        return result

    def _create_output_bar(self) -> Optional[IndicatorBar]:
        """创建输出的聚合 K 线"""
        if self._pending_open is None:
            return None
        return IndicatorBar(
            timestamp=self._pending_ts,
            open=self._pending_open,
            high=self._pending_high,
            low=self._pending_low,
            close=self._pending_close,
            volume=self._pending_volume,
            timeframe=self.target_tf,
        )

    def flush(self) -> Optional[IndicatorBar]:
        """强制输出当前未完成的聚合结果

        用于处理最后一批不完整的 K 线。

        Returns:
            如果有待处理数据则返回聚合结果，否则返回 None
        """
        result = self._create_output_bar()
        if result:
            self._reset()
        return result

    def _reset(self) -> None:
        """重置内部状态，准备下一个聚合周期"""
        self._current_period_start = None
        self._pending_open = None
        self._pending_high = float("-inf")
        self._pending_low = float("inf")
        self._pending_close = 0.0
        self._pending_volume = 0.0
        self._count = 0

    @property
    def pending_count(self) -> int:
        """当前待处理的 K 线数量"""
        return self._count

    @property
    def current_period_start(self) -> Optional[int]:
        """当前正在聚合的周期起始时间"""
        return self._current_period_start


def calculate_resample_ratio(source_tf: str, target_tf: str) -> int:
    """计算两个时间框架的比例

    Args:
        source_tf: 源时间框架，如 "15m"
        target_tf: 目标时间框架，如 "1h"

    Returns:
        比例值，如 4 (15m * 4 = 1h)

    Raises:
        ValueError: 当无法从源时间框架重采样到目标时间框架时
    """
    source_seconds = _tf_to_seconds(source_tf)
    target_seconds = _tf_to_seconds(target_tf)

    if target_seconds < source_seconds:
        raise ValueError(f"Cannot resample {source_tf} to smaller {target_tf}")
    if target_seconds % source_seconds != 0:
        raise ValueError(f"Cannot evenly resample {source_tf} to {target_tf}")

    return target_seconds // source_seconds


def _tf_to_seconds(tf: str) -> int:
    """时间框架转秒数

    支持的格式: 1m, 5m, 15m, 30m, 1h, 4h, 1d, 1w

    Args:
        tf: 时间框架字符串

    Returns:
        对应的秒数

    Raises:
        ValueError: 当时间框架格式不支持时
    """
    tf = tf.lower().strip()
    if not tf:
        raise ValueError("Empty timeframe")

    if tf.endswith("m"):
        return int(tf[:-1]) * 60
    if tf.endswith("h"):
        return int(tf[:-1]) * 3600
    if tf.endswith("d"):
        return int(tf[:-1]) * 86400
    if tf.endswith("w"):
        return int(tf[:-1]) * 7 * 86400

    raise ValueError(f"Unknown timeframe: {tf}")


def needs_resampling(source_tf: str, target_tf: str) -> bool:
    """判断是否需要重采样

    Args:
        source_tf: 源时间框架
        target_tf: 目标时间框架

    Returns:
        如果目标时间框架大于源时间框架，返回 True
    """
    try:
        source_seconds = _tf_to_seconds(source_tf)
        target_seconds = _tf_to_seconds(target_tf)
        return target_seconds > source_seconds
    except ValueError:
        return False


__all__ = [
    "OhlcvResampler",
    "calculate_resample_ratio",
    "needs_resampling",
]



from __future__ import annotations

from abc import ABC, abstractmethod


class BaseAccount(ABC):
    """账户基类"""

    def __init__(self, initial_capital: float):
        self._balance = float(initial_capital)

    @property
    def balance(self) -> float:
        """可用余额"""
        return self._balance

    @abstractmethod
    def apply_trade_result(self, trade_result: "TradeResult") -> "TradeResult":
        """根据交易结果更新资金"""
        raise NotImplementedError


class TradeResult:
    """交易结果占位"""

    def __init__(self, pnl: float = 0.0, fees: float = 0.0):
        self.pnl = pnl
        self.fees = fees
        self.balance_after = 0.0



from xqtrader.data.adapters.base import ExchangeAdapter, MarketType
from xqtrader.data.adapters.ccxt_adapter import CCXTAdapter
from xqtrader.data.adapters.binance import BinanceAdapter

__all__ = [
    "ExchangeAdapter",
    "MarketType",
    "CCXTAdapter",
    "BinanceAdapter",
]



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import Any, Tuple

from xqtrader.accounts.base import BaseAccount, TradeResult
from xqtrader.strategies.base import StrategySignal


class BaseTrader(ABC):
    """交易器基类"""

    @abstractmethod
    async def execute_trade(
        self,
        signal: StrategySignal,
        price: float,
        account: BaseAccount,
        config: Any,
    ) -> Tuple[TradeResult, Any]:
        raise NotImplementedError



from xqtrader.data.base import DataCenterService, MarketDataRequest
from xqtrader.data.cache import DataCenterCache, LRUCache
from xqtrader.data.circuit_breaker import CircuitBreaker, CircuitBreakerOpenError, CircuitState
from xqtrader.data.adapters import (
    ExchangeAdapter,
    MarketType,
    CCXTAdapter,
    BinanceAdapter,
)

__all__ = [
    "DataCenterService",
    "MarketDataRequest",
    "DataCenterCache",
    "LRUCache",
    "CircuitBreaker",
    "CircuitBreakerOpenError",
    "CircuitState",
    "ExchangeAdapter",
    "MarketType",
    "CCXTAdapter",
    "BinanceAdapter",
]



"""交易所适配器抽象基类

定义统一的交易所接口，支持多交易所扩展。
"""

from __future__ import annotations

from abc import ABC, abstractmethod
from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, Optional


class MarketType(str, Enum):
    """市场类型"""

    SPOT = "spot"
    FUTURES = "futures"  # USDT-M 永续
    DELIVERY = "delivery"  # COIN-M 交割


@dataclass
class OHLCVData:
    """标准化 OHLCV 数据结构"""

    timestamps: List[int]  # 毫秒时间戳
    open: List[float]
    high: List[float]
    low: List[float]
    close: List[float]
    volume: List[float]

    def __len__(self) -> int:
        return len(self.timestamps)

    def to_dict(self) -> Dict[str, List[float]]:
        return {
            "timestamps": self.timestamps,
            "open": self.open,
            "high": self.high,
            "low": self.low,
            "close": self.close,
            "volume": self.volume,
        }

    @classmethod
    def empty(cls) -> "OHLCVData":
        return cls(
            timestamps=[],
            open=[],
            high=[],
            low=[],
            close=[],
            volume=[],
        )


@dataclass
class TickerData:
    """行情数据"""

    symbol: str
    last_price: float
    bid_price: float
    ask_price: float
    volume_24h: float
    timestamp: int
    # 合约特有字段
    mark_price: Optional[float] = None
    index_price: Optional[float] = None
    funding_rate: Optional[float] = None


@dataclass
class OrderBookData:
    """订单簿数据"""

    symbol: str
    bids: List[List[float]]  # [[price, amount], ...]
    asks: List[List[float]]  # [[price, amount], ...]
    timestamp: int


class ExchangeAdapter(ABC):
    """交易所适配器抽象基类

    统一接口设计，支持：
    - 多交易所切换（Binance, OKX, Bybit 等）
    - 多市场类型（现货、U本位合约、币本位合约）
    - 同步/异步调用
    """

    def __init__(
        self,
        exchange_id: str,
        market_type: MarketType = MarketType.SPOT,
        api_key: Optional[str] = None,
        api_secret: Optional[str] = None,
        sandbox: bool = False,
    ) -> None:
        self.exchange_id = exchange_id
        self.market_type = market_type
        self.api_key = api_key
        self.api_secret = api_secret
        self.sandbox = sandbox

    @abstractmethod
    async def connect(self) -> None:
        """建立连接（初始化客户端）"""

    @abstractmethod
    async def close(self) -> None:
        """关闭连接"""

    async def __aenter__(self) -> "ExchangeAdapter":
        await self.connect()
        return self

    async def __aexit__(self, *args: Any) -> None:
        await self.close()

    # ==================== 市场数据 API ====================

    @abstractmethod
    async def fetch_ohlcv(
        self,
        symbol: str,
        interval: str,
        limit: int = 100,
        start_time: Optional[int] = None,
        end_time: Optional[int] = None,
    ) -> OHLCVData:
        """获取 K 线数据

        Args:
            symbol: 交易对，统一格式 "BTC/USDT"
            interval: K线周期 (1m, 5m, 15m, 1h, 4h, 1d)
            limit: 数据条数
            start_time: 开始时间（毫秒时间戳）
            end_time: 结束时间（毫秒时间戳）

        Returns:
            标准化的 OHLCV 数据
        """

    @abstractmethod
    async def fetch_ticker(self, symbol: str) -> TickerData:
        """获取最新行情"""

    @abstractmethod
    async def fetch_order_book(
        self, symbol: str, limit: int = 20
    ) -> OrderBookData:
        """获取订单簿"""

    async def fetch_mark_price(self, symbol: str) -> Optional[float]:
        """获取标记价格（合约专用）"""
        ticker = await self.fetch_ticker(symbol)
        return ticker.mark_price

    async def fetch_funding_rate(self, symbol: str) -> Optional[float]:
        """获取资金费率（合约专用）"""
        ticker = await self.fetch_ticker(symbol)
        return ticker.funding_rate

    # ==================== 工具方法 ====================

    @staticmethod
    def normalize_symbol(symbol: str, market_type: Optional["MarketType"] = None) -> str:
        """标准化交易对格式

        现货: BTCUSDT -> BTC/USDT
        永续: BTCUSDT -> BTC/USDT:USDT (CCXT 格式)
        """
        # 已经是标准格式
        if ":" in symbol:
            return symbol.upper()

        base_symbol = symbol.upper()

        # 处理已有斜杠的格式
        if "/" in base_symbol:
            if market_type in (MarketType.FUTURES, MarketType.DELIVERY):
                # 永续合约需要加后缀
                parts = base_symbol.split("/")
                if len(parts) == 2:
                    return f"{parts[0]}/{parts[1]}:{parts[1]}"
            return base_symbol

        # 解析无斜杠格式
        for quote in ["USDT", "USDC", "BUSD", "USD", "BTC", "ETH"]:
            if base_symbol.endswith(quote):
                base = base_symbol[: -len(quote)]
                if market_type in (MarketType.FUTURES, MarketType.DELIVERY):
                    return f"{base}/{quote}:{quote}"
                return f"{base}/{quote}"

        return base_symbol

    @staticmethod
    def denormalize_symbol(symbol: str, exchange_id: str = "binance") -> str:
        """反标准化: BTC/USDT -> BTCUSDT"""
        return symbol.replace("/", "")

    @staticmethod
    def normalize_interval(interval: str) -> str:
        """标准化时间间隔"""
        mapping = {
            "1min": "1m",
            "5min": "5m",
            "15min": "15m",
            "30min": "30m",
            "60min": "1h",
            "1hour": "1h",
            "4hour": "4h",
            "1day": "1d",
            "1week": "1w",
        }
        return mapping.get(interval.lower(), interval.lower())



"""数据缓存层

提供 K 线数据、行情数据的缓存机制，支持 TTL 过期和 LRU 淘汰。
"""

from __future__ import annotations

import time
from collections import OrderedDict
from dataclasses import dataclass
from typing import Any, Dict, Generic, Optional, TypeVar

T = TypeVar("T")


@dataclass
class CacheEntry(Generic[T]):
    """缓存条目"""

    data: T
    timestamp: float
    ttl: float

    def is_expired(self) -> bool:
        return time.time() - self.timestamp > self.ttl


class LRUCache(Generic[T]):
    """LRU 缓存实现

    支持：
    - TTL 过期
    - 最大容量限制
    - LRU 淘汰策略
    """

    def __init__(
        self,
        max_size: int = 1000,
        default_ttl: float = 300.0,
    ) -> None:
        self.max_size = max_size
        self.default_ttl = default_ttl
        self._cache: OrderedDict[str, CacheEntry[T]] = OrderedDict()
        self._hits = 0
        self._misses = 0

    def get(self, key: str) -> Optional[T]:
        """获取缓存数据"""
        if key not in self._cache:
            self._misses += 1
            return None

        entry = self._cache[key]

        if entry.is_expired():
            del self._cache[key]
            self._misses += 1
            return None

        # 移到末尾（最近访问）
        self._cache.move_to_end(key)
        self._hits += 1
        return entry.data

    def set(self, key: str, data: T, ttl: Optional[float] = None) -> None:
        """设置缓存数据"""
        if key in self._cache:
            del self._cache[key]

        # 检查容量，淘汰最旧的
        while len(self._cache) >= self.max_size:
            self._cache.popitem(last=False)

        self._cache[key] = CacheEntry(
            data=data,
            timestamp=time.time(),
            ttl=ttl or self.default_ttl,
        )

    def delete(self, key: str) -> bool:
        """删除缓存条目"""
        if key in self._cache:
            del self._cache[key]
            return True
        return False

    def clear(self) -> None:
        """清空缓存"""
        self._cache.clear()
        self._hits = 0
        self._misses = 0

    def clear_expired(self) -> int:
        """清除过期条目，返回清除数量"""
        expired_keys = [
            key for key, entry in self._cache.items() if entry.is_expired()
        ]
        for key in expired_keys:
            del self._cache[key]
        return len(expired_keys)

    @property
    def size(self) -> int:
        return len(self._cache)

    @property
    def hit_rate(self) -> float:
        total = self._hits + self._misses
        return self._hits / total if total > 0 else 0.0

    def stats(self) -> Dict[str, Any]:
        """获取缓存统计"""
        return {
            "size": self.size,
            "max_size": self.max_size,
            "hits": self._hits,
            "misses": self._misses,
            "hit_rate": self.hit_rate,
        }


class DataCenterCache:
    """数据中心缓存管理器

    为不同类型的数据维护独立的缓存：
    - K 线缓存（较长 TTL）
    - 行情缓存（较短 TTL）
    - 订单簿缓存（很短 TTL）
    """

    def __init__(
        self,
        kline_ttl: float = 60.0,
        ticker_ttl: float = 5.0,
        orderbook_ttl: float = 1.0,
        max_size: int = 1000,
    ) -> None:
        self.kline_cache: LRUCache[Dict[str, Any]] = LRUCache(
            max_size=max_size, default_ttl=kline_ttl
        )
        self.ticker_cache: LRUCache[Dict[str, Any]] = LRUCache(
            max_size=max_size // 2, default_ttl=ticker_ttl
        )
        self.orderbook_cache: LRUCache[Dict[str, Any]] = LRUCache(
            max_size=max_size // 4, default_ttl=orderbook_ttl
        )

    def _kline_key(
        self,
        symbol: str,
        interval: str,
        start_time: Optional[int] = None,
        end_time: Optional[int] = None,
        limit: int = 100,
    ) -> str:
        """生成 K 线缓存键"""
        return f"kline:{symbol}:{interval}:{start_time}:{end_time}:{limit}"

    def get_kline(
        self,
        symbol: str,
        interval: str,
        start_time: Optional[int] = None,
        end_time: Optional[int] = None,
        limit: int = 100,
    ) -> Optional[Dict[str, Any]]:
        """获取 K 线缓存"""
        key = self._kline_key(symbol, interval, start_time, end_time, limit)
        return self.kline_cache.get(key)

    def set_kline(
        self,
        symbol: str,
        interval: str,
        data: Dict[str, Any],
        start_time: Optional[int] = None,
        end_time: Optional[int] = None,
        limit: int = 100,
        ttl: Optional[float] = None,
    ) -> None:
        """设置 K 线缓存"""
        key = self._kline_key(symbol, interval, start_time, end_time, limit)
        self.kline_cache.set(key, data, ttl)

    def get_ticker(self, symbol: str) -> Optional[Dict[str, Any]]:
        """获取行情缓存"""
        return self.ticker_cache.get(f"ticker:{symbol}")

    def set_ticker(
        self, symbol: str, data: Dict[str, Any], ttl: Optional[float] = None
    ) -> None:
        """设置行情缓存"""
        self.ticker_cache.set(f"ticker:{symbol}", data, ttl)

    def get_orderbook(self, symbol: str) -> Optional[Dict[str, Any]]:
        """获取订单簿缓存"""
        return self.orderbook_cache.get(f"orderbook:{symbol}")

    def set_orderbook(
        self, symbol: str, data: Dict[str, Any], ttl: Optional[float] = None
    ) -> None:
        """设置订单簿缓存"""
        self.orderbook_cache.set(f"orderbook:{symbol}", data, ttl)

    def clear_all(self) -> None:
        """清空所有缓存"""
        self.kline_cache.clear()
        self.ticker_cache.clear()
        self.orderbook_cache.clear()

    def stats(self) -> Dict[str, Any]:
        """获取所有缓存统计"""
        return {
            "kline": self.kline_cache.stats(),
            "ticker": self.ticker_cache.stats(),
            "orderbook": self.orderbook_cache.stats(),
        }



"""增量指标计算引擎

基于 talipp 库提供 60+ 种技术指标的增量计算支持，支持多时间框架重采样。
"""

from __future__ import annotations

import time
from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional


@dataclass
class IndicatorRequirement:
    """指标需求配置

    Attributes:
        id: 指标唯一标识符
        type: 指标类型 (如 "macd", "rsi", "boll")
        timeframe: 时间框架 (如 "1h", "15m")
        params: 指标参数
    """

    id: str
    type: str
    timeframe: str
    params: Dict[str, Any] = field(default_factory=dict)


@dataclass
class IndicatorBar:
    """K 线数据

    Attributes:
        timestamp: 时间戳 (毫秒)
        open: 开盘价
        high: 最高价
        low: 最低价
        close: 收盘价
        volume: 成交量
        timeframe: 时间框架
    """

    timestamp: int
    open: float
    high: float
    low: float
    close: float
    volume: float
    timeframe: str


class BaseIncrementalIndicator(ABC):
    """增量指标基类

    所有指标实现必须继承此类并实现抽象方法。
    """

    def __init__(self, requirement: IndicatorRequirement) -> None:
        self.requirement = requirement
        self._bar_count = 0
        self._last_bar_ts: Optional[int] = None

    @property
    def bar_count(self) -> int:
        return self._bar_count

    @property
    def last_bar_ts(self) -> Optional[int]:
        return self._last_bar_ts

    @property
    @abstractmethod
    def warmup_period(self) -> int:
        raise NotImplementedError

    @property
    def is_warmed_up(self) -> bool:
        return self._bar_count >= self.warmup_period

    def update(self, bar: IndicatorBar) -> None:
        self._bar_count += 1
        self._last_bar_ts = bar.timestamp
        self._update(bar)

    @abstractmethod
    def _update(self, bar: IndicatorBar) -> None:
        raise NotImplementedError

    @abstractmethod
    def value(self) -> Any:
        raise NotImplementedError


class IndicatorEngine:
    """指标计算引擎

    管理多个指标的增量计算，支持多时间框架重采样。

    使用示例::

        engine = IndicatorEngine()
        engine.register_requirements({
            "macd_1h": {"type": "macd", "timeframe": "1h", "fast": 12, "slow": 26, "signal": 9},
            "rsi_15m": {"type": "rsi", "timeframe": "15m", "period": 14},
        }, source_timeframe="15m")

        for bar in bars:
            engine.update(bar)
            snapshot = engine.snapshot()
    """

    def __init__(self) -> None:
        self._requirements: Dict[str, IndicatorRequirement] = {}
        self._indicators: Dict[str, BaseIncrementalIndicator] = {}
        self._resamplers: Dict[str, Any] = {}  # OhlcvResampler by target timeframe
        self._source_timeframe: Optional[str] = None
        self._last_update_ts: Optional[int] = None
        self._last_bar_ts_by_timeframe: Dict[str, int] = {}

    def register_requirements(
        self,
        requirements: Dict[str, Dict[str, Any]],
        source_timeframe: Optional[str] = None,
    ) -> None:
        """注册指标需求

        Args:
            requirements: 指标配置字典，key 为指标 ID，value 为配置
            source_timeframe: 源数据时间框架（用于计算重采样比例）
        """
        from .resampler import OhlcvResampler, calculate_resample_ratio, needs_resampling

        self._source_timeframe = source_timeframe

        for req_id, spec in requirements.items():
            indicator_type = str(spec.get("type", "")).lower()
            timeframe = str(spec.get("timeframe", "")).lower()
            if not indicator_type or not timeframe:
                raise ValueError("indicator type and timeframe are required")

            params = {
                key: value
                for key, value in spec.items()
                if key not in {"type", "timeframe"}
            }
            requirement = IndicatorRequirement(
                id=req_id,
                type=indicator_type,
                timeframe=timeframe,
                params=params,
            )
            self._requirements[req_id] = requirement
            self._indicators[req_id] = self._build_indicator(requirement)

            # 设置重采样器（如果需要）
            target_tf = requirement.timeframe
            if source_timeframe and needs_resampling(source_timeframe, target_tf):
                if target_tf not in self._resamplers:
                    ratio = calculate_resample_ratio(source_timeframe, target_tf)
                    self._resamplers[target_tf] = OhlcvResampler(
                        source_tf=source_timeframe,
                        target_tf=target_tf,
                        ratio=ratio,
                    )

    def reset(self) -> None:
        """重置引擎状态"""
        self._requirements.clear()
        self._indicators.clear()
        self._resamplers.clear()
        self._source_timeframe = None
        self._last_update_ts = None
        self._last_bar_ts_by_timeframe.clear()

    def update(self, bar: IndicatorBar) -> None:
        """更新指标

        Args:
            bar: K 线数据
        """
        # 1. 更新与源 timeframe 匹配的指标
        for indicator in self._indicators.values():
            if indicator.requirement.timeframe == bar.timeframe:
                indicator.update(bar)

        # 2. 通过重采样器更新其他 timeframe 的指标
        for target_tf, resampler in self._resamplers.items():
            resampled = resampler.add(bar)
            if resampled:
                for indicator in self._indicators.values():
                    if indicator.requirement.timeframe == target_tf:
                        indicator.update(resampled)
                self._last_bar_ts_by_timeframe[target_tf] = resampled.timestamp

        self._last_update_ts = int(time.time() * 1000)
        self._last_bar_ts_by_timeframe[bar.timeframe] = bar.timestamp

    def warmup_from_ohlcv(self, ohlcv: Dict[str, List[float]], timeframe: str) -> None:
        """从 OHLCV 数据预热指标

        Args:
            ohlcv: OHLCV 数据字典
            timeframe: 时间框架
        """
        closes = ohlcv.get("close", [])
        if not closes:
            return

        opens = ohlcv.get("open", [])
        highs = ohlcv.get("high", [])
        lows = ohlcv.get("low", [])
        volumes = ohlcv.get("volume", [])
        timestamps = ohlcv.get("timestamps", [])

        for idx, close in enumerate(closes):
            bar = IndicatorBar(
                timestamp=int(timestamps[idx]) if idx < len(timestamps) else 0,
                open=opens[idx] if idx < len(opens) else close,
                high=highs[idx] if idx < len(highs) else close,
                low=lows[idx] if idx < len(lows) else close,
                close=close,
                volume=volumes[idx] if idx < len(volumes) else 0.0,
                timeframe=timeframe,
            )
            self.update(bar)

    def snapshot(self) -> Dict[str, Any]:
        """获取当前指标快照

        Returns:
            包含所有指标值和元数据的字典
        """
        by_type: Dict[str, Dict[str, Any]] = {}
        by_timeframe: Dict[str, Dict[str, Any]] = {}

        overall_warmup = True
        for req_id, indicator in self._indicators.items():
            indicator_type = indicator.requirement.type
            timeframe = indicator.requirement.timeframe
            value = indicator.value()

            by_type.setdefault(indicator_type, {})[req_id] = value
            by_timeframe.setdefault(timeframe, {}).setdefault(indicator_type, {})[
                req_id
            ] = value

            if not indicator.is_warmed_up:
                overall_warmup = False

        for timeframe, bucket in by_timeframe.items():
            warmup_state = all(
                indicator.is_warmed_up
                for indicator in self._indicators.values()
                if indicator.requirement.timeframe == timeframe
            )
            bucket["is_warmed_up"] = warmup_state
            bucket["bar_close_ts"] = self._last_bar_ts_by_timeframe.get(timeframe)

        snapshot = dict(by_type)
        snapshot["is_warmed_up"] = overall_warmup
        snapshot["timestamp"] = self._last_update_ts
        snapshot["bar_close_ts"] = (
            max(self._last_bar_ts_by_timeframe.values())
            if self._last_bar_ts_by_timeframe
            else None
        )
        snapshot["by_timeframe"] = by_timeframe
        return snapshot

    def _build_indicator(
        self, requirement: IndicatorRequirement
    ) -> BaseIncrementalIndicator:
        """使用 talipp 构建指标

        Args:
            requirement: 指标需求配置

        Returns:
            指标实例

        Raises:
            ValueError: 当指标类型不支持时
        """
        from .talipp_adapter import TALIPP_REGISTRY, TalippIndicator

        indicator_type = requirement.type.lower()
        if indicator_type not in TALIPP_REGISTRY:
            supported = ", ".join(sorted(TALIPP_REGISTRY.keys()))
            raise ValueError(
                f"Unsupported indicator type: {indicator_type}. "
                f"Supported types: {supported}"
            )

        cls, params_fn, warmup_fn, use_ohlcv, value_extractor = TALIPP_REGISTRY[
            indicator_type
        ]
        params = params_fn(requirement.params)
        warmup = warmup_fn(requirement.params)

        return TalippIndicator(
            requirement=requirement,
            indicator_class=cls,
            params=params,
            warmup=warmup,
            use_ohlcv=use_ohlcv,
            value_extractor=value_extractor,
        )


__all__ = [
    "IndicatorRequirement",
    "IndicatorBar",
    "BaseIncrementalIndicator",
    "IndicatorEngine",
]



from __future__ import annotations

from dataclasses import dataclass
from typing import List, Tuple

from xqtrader.accounts.base import BaseAccount, TradeResult
from xqtrader.accounts.futures import FuturesSimulatedAccount, PositionSide
from xqtrader.traders.base import BaseTrader
from xqtrader.strategies.base import StrategySignal


@dataclass
class FuturesBacktestConfig:
    symbol: str
    interval: str
    leverage: int = 10
    position_size_pct: float = 0.1
    taker_fee: float = 0.0004
    maker_fee: float = 0.0002
    slippage: float = 0.0005
    maintenance_margin_ratio: float = 0.004
    funding_rate_interval: int = 28800
    start_time: int | None = None
    end_time: int | None = None


@dataclass
class FuturesPosition:
    symbol: str
    side: PositionSide
    entry_price: float
    size: float
    leverage: int
    margin: float
    entry_fee: float = 0.0
    liquidation_price: float = 0.0


class HedgePositionManager:
    def __init__(self, symbol: str):
        self.symbol = symbol
        self.long_position: FuturesPosition | None = None
        self.short_position: FuturesPosition | None = None

    def get_position(self, side: PositionSide) -> FuturesPosition | None:
        return self.long_position if side == PositionSide.LONG else self.short_position

    def set_position(self, position: FuturesPosition | None, side: PositionSide) -> None:
        if side == PositionSide.LONG:
            self.long_position = position
        else:
            self.short_position = position

    def has_position(self, side: PositionSide | None = None) -> bool:
        if side is None:
            return self.long_position is not None or self.short_position is not None
        return self.get_position(side) is not None

    def get_total_margin(self) -> float:
        margin = 0.0
        if self.long_position:
            margin += self.long_position.margin
        if self.short_position:
            margin += self.short_position.margin
        return margin


class FuturesTrader(BaseTrader):
    async def execute_trade(
        self,
        signal: StrategySignal,
        price: float,
        account: BaseAccount,
        config: FuturesBacktestConfig,
        position_manager: HedgePositionManager | None = None,
    ) -> Tuple[TradeResult, List[dict]]:
        if position_manager is None:
            position_manager = HedgePositionManager(config.symbol)
        if not isinstance(account, FuturesSimulatedAccount):
            raise TypeError("FuturesTrader requires FuturesSimulatedAccount")
        # 极简实现：仅支持开仓/平仓，未实现止盈止损
        action = signal.action.upper()
        side = PositionSide.LONG if action in {"LONG", "BUY", "OPEN_LONG"} else PositionSide.SHORT
        records: List[dict] = []

        if action in {"LONG", "BUY", "OPEN_LONG", "SHORT", "SELL", "OPEN_SHORT"}:
            margin = account.balance * config.position_size_pct
            nominal = margin * config.leverage
            trade_price = price * (1 + config.slippage)
            size = nominal / trade_price
            fee = trade_price * size * config.taker_fee
            account.lock_margin(margin, side)
            account.apply_fee(fee)
            pos = FuturesPosition(
                symbol=config.symbol,
                side=side,
                entry_price=trade_price,
                size=size,
                leverage=config.leverage,
                margin=margin,
                entry_fee=fee,
                liquidation_price=trade_price * (1 - config.maintenance_margin_ratio) if side == PositionSide.LONG else trade_price * (1 + config.maintenance_margin_ratio),
            )
            position_manager.set_position(pos, side)
            records.append({"action": action, "side": side.value, "price": trade_price, "size": size})
            result = TradeResult(pnl=-fee, fees=fee)
            account.apply_trade_result(result)
            return result, records

        if action in {"CLOSE_LONG", "CLOSE_SHORT", "CLOSE"}:
            sides = [PositionSide.LONG, PositionSide.SHORT] if action == "CLOSE" else [PositionSide.LONG if "LONG" in action else PositionSide.SHORT]
            total_pnl = 0.0
            total_fee = 0.0
            for s in sides:
                pos = position_manager.get_position(s)
                if not pos:
                    continue
                exit_price = price * (1 - config.slippage if s == PositionSide.LONG else 1 + config.slippage)
                realized = (exit_price - pos.entry_price) * pos.size if s == PositionSide.LONG else (pos.entry_price - exit_price) * pos.size
                fee = exit_price * pos.size * config.taker_fee
                pnl = realized - fee - pos.entry_fee
                account.release_margin(pos.margin, s)
                account.apply_fee(fee)
                total_pnl += pnl
                total_fee += fee
                records.append({"action": action, "side": s.value, "price": exit_price, "size": pos.size, "pnl": pnl})
                position_manager.set_position(None, s)
            result = TradeResult(pnl=total_pnl, fees=total_fee)
            account.apply_trade_result(result)
            return result, records

        return TradeResult(), records



from __future__ import annotations

from dataclasses import dataclass

from .base import BaseAccount, TradeResult


@dataclass
class SimulatedAccount(BaseAccount):
    """简单模拟账户 - 用于事件合约回测"""

    def __init__(self, initial_capital: float):
        super().__init__(initial_capital)

    def apply_trade_result(self, trade_result: TradeResult) -> TradeResult:
        self._balance += trade_result.pnl
        trade_result.balance_after = self._balance
        return trade_result



from xqtrader._version import __version__
from xqtrader.strategies.base import (
    BaseStrategy,
    StrategyContext,
    StrategyResult,
    StrategySignal,
    StrategyDataRequirements,
    StrategyLoader,
)
from xqtrader.engine.base import BaseEngine, ExecutionConfig, ExecutionEvent, ExecutionMode
from xqtrader.engine.backtest import BacktestEngine, BacktestConfig
from xqtrader.accounts.simulated import SimulatedAccount
from xqtrader.accounts.futures import FuturesSimulatedAccount, PositionSide
from xqtrader.traders.events import EventsTrader, EventsBacktestConfig
from xqtrader.traders.futures import FuturesTrader, FuturesBacktestConfig, HedgePositionManager
from xqtrader.data.base import DataCenterService, MarketDataRequest
from xqtrader.risk.base import RiskManager, RiskConfig, RiskLevel, RiskAction
from xqtrader.reports.base import BacktestReport, TradeRecord, EquityPoint, ReportGenerator, ReportCollector

__all__ = [
    "__version__",
    "BaseStrategy",
    "StrategyContext",
    "StrategyResult",
    "StrategySignal",
    "StrategyDataRequirements",
    "StrategyLoader",
    "BaseEngine",
    "ExecutionConfig",
    "ExecutionEvent",
    "ExecutionMode",
    "BacktestEngine",
    "BacktestConfig",
    "SimulatedAccount",
    "FuturesSimulatedAccount",
    "PositionSide",
    "EventsTrader",
    "EventsBacktestConfig",
    "FuturesTrader",
    "FuturesBacktestConfig",
    "HedgePositionManager",
    "DataCenterService",
    "MarketDataRequest",
    "RiskManager",
    "RiskConfig",
    "RiskLevel",
    "RiskAction",
    "BacktestReport",
    "TradeRecord",
    "EquityPoint",
    "ReportGenerator",
    "ReportCollector",
]



from __future__ import annotations

from dataclasses import dataclass
from typing import Tuple

from xqtrader.accounts.base import BaseAccount, TradeResult
from xqtrader.traders.base import BaseTrader
from xqtrader.strategies.base import StrategySignal


@dataclass
class EventsBacktestConfig:
    symbol: str
    interval: str
    investment_amount: float
    payout_multiplier: float = 1.8
    start_time: int | None = None
    end_time: int | None = None


class EventsTrader(BaseTrader):
    DEFAULT_PAYOUT_MULTIPLIER = 1.8

    def _normalize_action(self, action: str) -> str:
        action_upper = action.upper()
        if action_upper in {"UP", "DOWN", "HOLD"}:
            return action_upper
        if action_upper in {"LONG", "BUY"}:
            return "UP"
        if action_upper in {"SHORT", "SELL"}:
            return "DOWN"
        return action_upper

    async def execute_trade(
        self,
        signal: StrategySignal,
        price: float,
        account: BaseAccount,
        config: EventsBacktestConfig,
    ) -> Tuple[TradeResult, dict]:
        normalized_action = self._normalize_action(signal.action)
        if normalized_action not in {"UP", "DOWN", "HOLD"}:
            normalized_action = "HOLD"

        if normalized_action == "HOLD":
            result = TradeResult()
            record = {
                "symbol": config.symbol,
                "action": normalized_action,
                "entry_price": price,
                "payout_multiplier": config.payout_multiplier,
                "win": None,
                "skipped": True,
            }
            return result, record

        invest = signal.quantity or config.investment_amount
        payout_multiplier = config.payout_multiplier or self.DEFAULT_PAYOUT_MULTIPLIER
        # 简化：胜负由信号 action 决定（UP 视为看涨，DOWN 视为看跌）
        win = normalized_action == "UP"
        pnl = invest * (payout_multiplier - 1) if win else -invest
        result = TradeResult(pnl=pnl, fees=0.0)
        account.apply_trade_result(result)
        record = {
            "symbol": config.symbol,
            "action": normalized_action,
            "entry_price": price,
            "payout_multiplier": payout_multiplier,
            "win": win,
        }
        return result, record



from __future__ import annotations

from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from datetime import datetime
from pathlib import Path
from typing import Any, Callable, Dict, List, Optional


@dataclass
class StrategyDataRequirements:
    use_time_range: bool = True
    min_bars: int = 0
    prefer_closed_bar: bool = False
    extra_seconds: int = 0
    warmup_periods: int = 50
    max_timeframe_required: Optional[str] = None


@dataclass
class StrategySignal:
    """策略信号

    事件合约建议使用 UP/DOWN/HOLD，永续合约使用 LONG/SHORT/CLOSE_*。
    事件合约交易器会将 LONG/SHORT/BUY/SELL 兼容映射为 UP/DOWN。
    """
    action: str
    symbol: str
    quantity: float = 0.0
    price: Optional[float] = None
    stop_loss: Optional[float] = None
    take_profit: Optional[float] = None
    confidence: float = 1.0
    reason: str = ""


@dataclass
class StrategyContext:
    symbol: str
    interval: str
    current_time: datetime
    market_data: Dict[str, List[float]]
    indicators: Dict[str, Any] = field(default_factory=dict)
    incremental_indicators: Dict[str, Any] = field(default_factory=dict)
    account_balance: float = 0.0
    current_positions: Dict[str, float] = field(default_factory=dict)


@dataclass
class StrategyResult:
    signals: List[StrategySignal]
    indicators: Dict[str, Any]
    metadata: Dict[str, Any]
    execution_time: float
    success: bool
    error_message: Optional[str] = None


class BaseStrategy(ABC):
    """策略基类 - 所有用户策略必须继承此类"""

    def __init__(
        self,
        name: str,
        version: str = "1.0.0",
        description: str = "",
        tags: Optional[List[str]] = None,
    ) -> None:
        self.name = name
        self.version = version
        self.description = description
        self.tags = tags or []

    @abstractmethod
    def execute(self, context: StrategyContext) -> StrategyResult:
        """策略执行入口 - 必须实现"""

    def get_config(self) -> Dict[str, Any]:
        return {}

    def get_data_requirements(
        self, interval: str, config: Optional[Dict[str, Any]] = None
    ) -> StrategyDataRequirements:
        return StrategyDataRequirements()

    def get_indicator_requirements(self) -> Dict[str, Dict[str, Any]]:
        return {}

    def create_signal(
        self,
        action: str,
        symbol: str,
        quantity: float = 0.0,
        **kwargs: Any,
    ) -> StrategySignal:
        return StrategySignal(action=action, symbol=symbol, quantity=quantity, **kwargs)


@dataclass
class LoadedStrategyEntry:
    factory: Callable[[], BaseStrategy]
    module_name: str
    file_path: str
    metadata: Dict[str, Any]


class StrategyLoader:
    """策略加载器 - 动态加载 Python 策略文件"""

    def __init__(self, strategies_dir: str) -> None:
        self.strategies_dir = Path(strategies_dir)
        self.loaded_strategies: Dict[str, LoadedStrategyEntry] = {}

    def load_all_strategies(self) -> Dict[str, LoadedStrategyEntry]:
        for file_path in self.strategies_dir.glob("*.py"):
            self.load_strategy_from_file(str(file_path))
        return self.loaded_strategies

    def load_strategy_from_file(self, file_path: str) -> Optional[BaseStrategy]:
        import importlib.util
        import sys

        path = Path(file_path)
        if not path.exists():
            return None

        module_name = f"strategy_{path.stem}"
        spec = importlib.util.spec_from_file_location(module_name, file_path)
        if spec is None or spec.loader is None:
            return None

        module = importlib.util.module_from_spec(spec)
        sys.modules[module_name] = module
        spec.loader.exec_module(module)

        strategy_cls = None
        strategy_instance = getattr(module, "strategy_instance", None)
        if strategy_instance and isinstance(strategy_instance, BaseStrategy):
            strategy_cls = strategy_instance.__class__
            def factory() -> BaseStrategy:
                try:
                    return strategy_cls(  # type: ignore[misc]
                        **{k: v for k, v in getattr(strategy_instance, "__dict__", {}).items() if k in {"name", "version", "description", "tags"}}
                    )
                except TypeError:
                    return strategy_cls(name=getattr(strategy_instance, "name", path.stem))  # type: ignore[misc]
        else:
            for attr in module.__dict__.values():
                if isinstance(attr, type) and issubclass(attr, BaseStrategy) and attr is not BaseStrategy:
                    strategy_cls = attr
                    break
            if strategy_cls is None:
                return None
            def factory() -> BaseStrategy:
                try:
                    return strategy_cls()  # type: ignore[misc]
                except TypeError:
                    return strategy_cls(name=path.stem)  # type: ignore[misc]

        if strategy_cls is None:
            return None

        instance = factory()
        metadata = {
            "name": getattr(instance, "name", path.stem),
            "version": getattr(instance, "version", "1.0.0"),
            "description": getattr(instance, "description", ""),
        }

        self.loaded_strategies[path.stem] = LoadedStrategyEntry(
            factory=factory, module_name=module_name, file_path=str(path), metadata=metadata
        )
        return instance

    def execute_strategy(self, strategy_id: str, context: StrategyContext) -> Optional[StrategyResult]:
        entry = self.loaded_strategies.get(strategy_id)
        if entry is None:
            return None
        strategy = entry.factory()
        return strategy.execute(context)

    def reload_strategy_from_file(self, file_path: str) -> bool:
        path = Path(file_path)
        if path.stem in self.loaded_strategies:
            del self.loaded_strategies[path.stem]
        return self.load_strategy_from_file(file_path) is not None



"""qt backtest - 策略回测命令"""

from __future__ import annotations

import asyncio
import sys
from datetime import datetime, timezone
from pathlib import Path
from typing import Optional

import click

from cli.console import console, print_report_summary, print_trade_event


def _parse_time_ms(value: str) -> int:
    """将日期字符串解析为毫秒时间戳"""
    for fmt in ("%Y-%m-%d %H:%M:%S", "%Y-%m-%d %H:%M", "%Y-%m-%d"):
        try:
            dt = datetime.strptime(value, fmt).replace(tzinfo=timezone.utc)
            return int(dt.timestamp() * 1000)
        except ValueError:
            continue
    raise click.BadParameter(f"无法解析日期: {value}，支持格式: YYYY-MM-DD [HH:MM[:SS]]")


@click.command()
@click.option(
    "--strategy", "-s",
    required=True,
    type=click.Path(exists=True, dir_okay=False),
    help="策略文件路径 (.py)",
)
@click.option("--symbol", required=True, default="BTC/USDT", show_default=True, help="交易对")
@click.option("--interval", "-i", default="1m", show_default=True, help="K线周期")
@click.option("--start", required=True, help="开始时间 (YYYY-MM-DD [HH:MM])")
@click.option("--end", required=True, help="结束时间 (YYYY-MM-DD [HH:MM])")
@click.option("--capital", default=10000.0, show_default=True, help="初始资金")
@click.option(
    "--contract",
    type=click.Choice(["futures", "events"], case_sensitive=False),
    default="futures",
    show_default=True,
    help="合约类型",
)
@click.option("--report", "-r", type=click.Path(), default=None, help="报告输出路径 (.json)")
@click.option("--no-cache", is_flag=True, default=False, help="禁用数据缓存")
@click.option("--verbose", "-v", is_flag=True, default=False, help="显示详细交易日志")
def backtest(
    strategy: str,
    symbol: str,
    interval: str,
    start: str,
    end: str,
    capital: float,
    contract: str,
    report: Optional[str],
    no_cache: bool,
    verbose: bool,
) -> None:
    """运行策略回测"""
    try:
        asyncio.run(_run_backtest(
            strategy_path=strategy,
            symbol=symbol,
            interval=interval,
            start=start,
            end=end,
            capital=capital,
            contract=contract,
            report_path=report,
            enable_cache=not no_cache,
            verbose=verbose,
        ))
    except KeyboardInterrupt:
        console.print("\n[yellow]回测已中断[/yellow]")
        sys.exit(1)
    except Exception as e:
        console.print(f"[red]回测失败: {e}[/red]")
        sys.exit(1)


async def _run_backtest(
    strategy_path: str,
    symbol: str,
    interval: str,
    start: str,
    end: str,
    capital: float,
    contract: str,
    report_path: Optional[str],
    enable_cache: bool,
    verbose: bool,
) -> None:
    from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn, TaskProgressColumn

    from xqtrader.engine.backtest import BacktestConfig, BacktestEngine
    from xqtrader.reports.base import ReportCollector, ReportGenerator
    from xqtrader.strategies.base import StrategyLoader

    # 加载策略
    loader = StrategyLoader(str(Path(strategy_path).parent))
    strategy_instance = loader.load_strategy_from_file(strategy_path)
    if strategy_instance is None:
        console.print(f"[red]无法加载策略文件: {strategy_path}[/red]")
        console.print("[dim]确保文件包含继承 BaseStrategy 的策略类[/dim]")
        sys.exit(1)

    console.print(f"[green]策略已加载:[/green] {strategy_instance.name} v{strategy_instance.version}")

    start_ms = _parse_time_ms(start)
    end_ms = _parse_time_ms(end)

    config = BacktestConfig(
        symbol=symbol,
        interval=interval,
        initial_capital=capital,
        contract_type=contract,
        start_time=start_ms,
        end_time=end_ms,
        enable_cache=enable_cache,
    )

    collector = ReportCollector(
        strategy_name=strategy_instance.name,
        symbol=symbol,
        interval=interval,
        initial_capital=capital,
        start_time=start_ms,
        end_time=end_ms,
    )

    engine = BacktestEngine()
    tick_count = 0
    trade_count = 0

    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        BarColumn(),
        TaskProgressColumn(),
        TextColumn("{task.fields[info]}"),
        console=console,
        transient=True,
    ) as progress:
        task = progress.add_task("回测中...", total=None, info="")

        async for event in engine.run(strategy_instance, config):
            collector.collect(event)

            if event.event_type == "tick":
                tick_count += 1
                bar = event.data.get("bar", {})
                progress.update(
                    task,
                    description=f"回测中... ({tick_count} bars)",
                    info=f"价格: {bar.get('close', 'N/A')}",
                )

            elif event.event_type == "trade":
                records = event.data.get("records", [])
                record = event.data.get("record")
                if records or (record and not record.get("skipped")):
                    trade_count += 1
                    if verbose:
                        progress.stop()
                        print_trade_event(trade_count, event.data)
                        progress.start()

            elif event.event_type == "complete":
                progress.update(task, description="回测完成", info="")

    # 构建报告
    report = collector.build()

    # 打印摘要
    print_report_summary(report)

    # 导出报告
    if report_path is None:
        report_path = f"report_{strategy_instance.name}_{symbol.replace('/', '_')}_{contract}.json"

    ReportGenerator.export(report, format="json", path=report_path)
    console.print(f"\n[dim]报告已导出: {report_path}[/dim]")



from xqtrader.strategies.base import (
    BaseStrategy,
    StrategyContext,
    StrategyDataRequirements,
    StrategyLoader,
    StrategyResult,
    StrategySignal,
)

__all__ = [
    "BaseStrategy",
    "StrategyContext",
    "StrategyDataRequirements",
    "StrategyLoader",
    "StrategyResult",
    "StrategySignal",
]



"""数据中心服务

统一数据获取入口，整合：
- 交易所适配器（CCXT/Binance）
- 缓存层（LRU + TTL）
- 熔断器（故障保护）
"""

from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Type

from xqtrader.data.adapters.base import (
    ExchangeAdapter,
    MarketType,
    OHLCVData,
    OrderBookData,
    TickerData,
)
from xqtrader.data.adapters.ccxt_adapter import CCXTAdapter, CCXT_AVAILABLE
from xqtrader.data.adapters.binance import BinanceAdapter
from xqtrader.data.cache import DataCenterCache
from xqtrader.data.circuit_breaker import CircuitBreaker, CircuitBreakerOpenError


@dataclass
class MarketDataRequest:
    """市场数据请求"""

    symbol: str
    interval: str
    exchange: str = "binance"
    market_type: str = "spot"  # spot | futures | delivery
    limit: int = 100
    start_time: Optional[int] = None
    end_time: Optional[int] = None


class DataCenterService:
    """数据中心服务 - 统一数据获取入口

    特性：
    - 多交易所支持（通过 CCXT）
    - 自动缓存（可配置 TTL）
    - 熔断保护（失败自动熔断）
    - 支持现货和合约市场

    Example:
        # 基础用法
        async with DataCenterService() as dc:
            data = await dc.get_market_data(
                MarketDataRequest(symbol="BTC/USDT", interval="1h")
            )

        # 永续合约
        async with DataCenterService(market_type=MarketType.FUTURES) as dc:
            data = await dc.get_market_data(
                MarketDataRequest(symbol="BTC/USDT", interval="1h", market_type="futures")
            )
    """

    # 支持的交易所映射
    EXCHANGE_ADAPTERS: Dict[str, Type[ExchangeAdapter]] = {
        "binance": BinanceAdapter,
        # 未来扩展
        # "okx": OKXAdapter,
        # "bybit": BybitAdapter,
    }

    def __init__(
        self,
        exchange: str = "binance",
        market_type: MarketType = MarketType.SPOT,
        api_key: Optional[str] = None,
        api_secret: Optional[str] = None,
        enable_cache: bool = True,
        cache_ttl: float = 60.0,
        enable_circuit_breaker: bool = True,
        sandbox: bool = False,
    ) -> None:
        self.exchange_id = exchange.lower()
        self.market_type = market_type
        self.api_key = api_key
        self.api_secret = api_secret
        self.sandbox = sandbox

        # 缓存
        self._cache: Optional[DataCenterCache] = None
        if enable_cache:
            self._cache = DataCenterCache(kline_ttl=cache_ttl)

        # 熔断器
        self._circuit_breaker: Optional[CircuitBreaker] = None
        if enable_circuit_breaker:
            self._circuit_breaker = CircuitBreaker(
                name=f"{exchange}_{market_type.value}",
                failure_threshold=5,
                timeout=30.0,
            )

        # 适配器实例
        self._adapter: Optional[ExchangeAdapter] = None

    def _get_adapter_class(self) -> Type[ExchangeAdapter]:
        """获取适配器类"""
        if self.exchange_id in self.EXCHANGE_ADAPTERS:
            return self.EXCHANGE_ADAPTERS[self.exchange_id]
        # 通用 CCXT 适配器
        return CCXTAdapter

    async def connect(self) -> None:
        """建立连接"""
        if not CCXT_AVAILABLE:
            raise ImportError(
                "ccxt is required. Install with: pip install ccxt"
            )

        adapter_class = self._get_adapter_class()

        if adapter_class == BinanceAdapter:
            self._adapter = BinanceAdapter(
                market_type=self.market_type,
                api_key=self.api_key,
                api_secret=self.api_secret,
                sandbox=self.sandbox,
            )
        else:
            self._adapter = CCXTAdapter(
                exchange_id=self.exchange_id,
                market_type=self.market_type,
                api_key=self.api_key,
                api_secret=self.api_secret,
                sandbox=self.sandbox,
            )

        await self._adapter.connect()

    async def close(self) -> None:
        """关闭连接"""
        if self._adapter:
            await self._adapter.close()
            self._adapter = None

    async def __aenter__(self) -> "DataCenterService":
        await self.connect()
        return self

    async def __aexit__(self, *args: Any) -> None:
        await self.close()

    @property
    def adapter(self) -> ExchangeAdapter:
        """获取底层适配器"""
        if self._adapter is None:
            raise RuntimeError("DataCenterService not connected")
        return self._adapter

    def _check_circuit_breaker(self) -> None:
        """检查熔断器状态"""
        if self._circuit_breaker and not self._circuit_breaker.allow_request():
            raise CircuitBreakerOpenError(self._circuit_breaker)

    def _record_success(self) -> None:
        """记录成功"""
        if self._circuit_breaker:
            self._circuit_breaker.record_success()

    def _record_failure(self) -> None:
        """记录失败"""
        if self._circuit_breaker:
            self._circuit_breaker.record_failure()

    async def get_market_data(
        self, request: MarketDataRequest
    ) -> Dict[str, Any]:
        """获取市场数据（带缓存）

        返回格式：
        {
            "ohlcv": {
                "open": [...], "high": [...], "low": [...],
                "close": [...], "volume": [...], "timestamps": [...]
            },
            "metadata": {
                "symbol": "...",
                "interval": "...",
                "count": 100,
                "exchange": "binance"
            }
        }
        """
        symbol = ExchangeAdapter.normalize_symbol(request.symbol, self.market_type)

        # 尝试从缓存获取
        if self._cache:
            cached = self._cache.get_kline(
                symbol,
                request.interval,
                request.start_time,
                request.end_time,
                request.limit,
            )
            if cached:
                return cached

        # 检查熔断器
        self._check_circuit_breaker()

        try:
            ohlcv = await self.adapter.fetch_ohlcv(
                symbol,
                request.interval,
                limit=request.limit,
                start_time=request.start_time,
                end_time=request.end_time,
            )

            result = {
                "ohlcv": ohlcv.to_dict(),
                "metadata": {
                    "symbol": symbol,
                    "interval": request.interval,
                    "count": len(ohlcv),
                    "exchange": self.exchange_id,
                    "market_type": self.market_type.value,
                },
            }

            # 缓存结果
            if self._cache:
                self._cache.set_kline(
                    symbol,
                    request.interval,
                    result,
                    request.start_time,
                    request.end_time,
                    request.limit,
                )

            self._record_success()
            return result

        except Exception as e:
            self._record_failure()
            raise

    async def get_historical_klines_batch(
        self,
        request: MarketDataRequest,
        batch_size: int = 1000,
    ) -> Dict[str, Any]:
        """批量获取历史 K 线数据"""
        if not request.start_time or not request.end_time:
            raise ValueError("start_time and end_time required for batch fetch")

        self._check_circuit_breaker()

        try:
            if isinstance(self.adapter, CCXTAdapter):
                ohlcv = await self.adapter.fetch_ohlcv_batch(
                    request.symbol,
                    request.interval,
                    request.start_time,
                    request.end_time,
                    batch_size=batch_size,
                )
            else:
                ohlcv = await self.adapter.fetch_ohlcv(
                    request.symbol,
                    request.interval,
                    limit=request.limit,
                    start_time=request.start_time,
                    end_time=request.end_time,
                )

            self._record_success()

            return {
                "ohlcv": ohlcv.to_dict(),
                "metadata": {
                    "symbol": request.symbol,
                    "interval": request.interval,
                    "count": len(ohlcv),
                    "exchange": self.exchange_id,
                },
            }
        except Exception as e:
            self._record_failure()
            raise

    async def get_ticker(self, symbol: str) -> Dict[str, Any]:
        """获取最新行情"""
        symbol = ExchangeAdapter.normalize_symbol(symbol, self.market_type)

        # 尝试从缓存获取
        if self._cache:
            cached = self._cache.get_ticker(symbol)
            if cached:
                return cached

        self._check_circuit_breaker()

        try:
            ticker = await self.adapter.fetch_ticker(symbol)
            result = {
                "symbol": ticker.symbol,
                "last_price": ticker.last_price,
                "bid_price": ticker.bid_price,
                "ask_price": ticker.ask_price,
                "volume_24h": ticker.volume_24h,
                "timestamp": ticker.timestamp,
                "mark_price": ticker.mark_price,
                "index_price": ticker.index_price,
                "funding_rate": ticker.funding_rate,
            }

            if self._cache:
                self._cache.set_ticker(symbol, result)

            self._record_success()
            return result

        except Exception as e:
            self._record_failure()
            raise

    async def get_order_book(
        self, symbol: str, limit: int = 20
    ) -> Dict[str, Any]:
        """获取订单簿"""
        symbol = ExchangeAdapter.normalize_symbol(symbol, self.market_type)

        if self._cache:
            cached = self._cache.get_orderbook(symbol)
            if cached:
                return cached

        self._check_circuit_breaker()

        try:
            order_book = await self.adapter.fetch_order_book(symbol, limit)
            result = {
                "symbol": order_book.symbol,
                "bids": order_book.bids,
                "asks": order_book.asks,
                "timestamp": order_book.timestamp,
            }

            if self._cache:
                self._cache.set_orderbook(symbol, result)

            self._record_success()
            return result

        except Exception as e:
            self._record_failure()
            raise

    def cache_stats(self) -> Dict[str, Any]:
        """获取缓存统计"""
        if self._cache:
            return self._cache.stats()
        return {}

    def circuit_breaker_stats(self) -> Dict[str, Any]:
        """获取熔断器统计"""
        if self._circuit_breaker:
            stats = self._circuit_breaker.stats()
            return {
                "state": stats.state.value,
                "failure_count": stats.failure_count,
                "success_count": stats.success_count,
            }
        return {}



from __future__ import annotations

from dataclasses import dataclass
from enum import Enum

from .base import BaseAccount, TradeResult


class PositionSide(str, Enum):
    LONG = "long"
    SHORT = "short"


@dataclass
class FuturesSimulatedAccount(BaseAccount):
    """永续合约模拟账户 - 支持双向持仓"""

    def __init__(self, initial_capital: float):
        super().__init__(initial_capital)
        self._long_margin_locked = 0.0
        self._short_margin_locked = 0.0

    @property
    def margin_locked(self) -> float:
        return self._long_margin_locked + self._short_margin_locked

    @property
    def wallet_balance(self) -> float:
        return self._balance + self.margin_locked

    def lock_margin(self, amount: float, side: PositionSide):
        if amount > self._balance:
            raise ValueError("Insufficient funds")
        self._balance -= amount
        if side == PositionSide.LONG:
            self._long_margin_locked += amount
        else:
            self._short_margin_locked += amount

    def release_margin(self, amount: float, side: PositionSide):
        if side == PositionSide.LONG:
            self._long_margin_locked -= amount
        else:
            self._short_margin_locked -= amount
        self._balance += amount

    def apply_fee(self, fee: float):
        self._balance -= fee

    def apply_pnl(self, pnl: float):
        self._balance += pnl

    def apply_trade_result(self, trade_result: TradeResult) -> TradeResult:
        self._balance += trade_result.pnl
        trade_result.balance_after = self._balance
        return trade_result



from __future__ import annotations

from abc import ABC, abstractmethod
from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Any, AsyncGenerator, Dict, Optional

from xqtrader.strategies.base import BaseStrategy, StrategyContext, StrategyResult


class ExecutionMode(str, Enum):
    BACKTEST = "backtest"
    PAPER = "paper"
    LIVE = "live"


@dataclass
class ExecutionEvent:
    event_type: str  # tick, trade, progress, complete, error
    data: Dict[str, Any]
    timestamp: datetime


@dataclass
class ExecutionConfig:
    symbol: str
    interval: str
    start_time: Optional[int] = None
    end_time: Optional[int] = None


class BaseEngine(ABC):
    def __init__(self, mode: ExecutionMode) -> None:
        self.mode = mode

    @abstractmethod
    async def run(
        self,
        strategy: BaseStrategy,
        config: ExecutionConfig,
        progress_callback: Optional[Any] = None,
    ) -> AsyncGenerator[ExecutionEvent, None]:
        if False:
            yield ExecutionEvent(event_type="noop", data={}, timestamp=datetime.utcnow())
        raise NotImplementedError

    async def _emit(
        self, event_type: str, data: Dict[str, Any]
    ) -> ExecutionEvent:
        return ExecutionEvent(event_type=event_type, data=data, timestamp=datetime.utcnow())


__all__ = [
    "ExecutionMode",
    "ExecutionEvent",
    "ExecutionConfig",
    "BaseEngine",
]



from __future__ import annotations

from dataclasses import dataclass
from enum import Enum
from typing import Any, Dict, List, Optional

from xqtrader.accounts.base import BaseAccount


class RiskLevel(str, Enum):
    NORMAL = "normal"
    WARNING = "warning"
    CRITICAL = "critical"


class RiskAction(str, Enum):
    NONE = "none"
    WARN = "warn"
    STOP_TRADING = "stop"
    FORCE_CLOSE = "force_close"


@dataclass
class RiskRule:
    name: str
    level: RiskLevel
    threshold: float
    action: RiskAction
    description: str = ""


@dataclass
class RiskConfig:
    max_daily_loss_pct: float = 0.05
    max_drawdown_pct: float = 0.15
    warning_ratio: float = 0.7


DEFAULT_RISK_RULES: List[RiskRule] = [
    RiskRule(
        name="daily_loss_warning",
        level=RiskLevel.WARNING,
        threshold=0.035,
        action=RiskAction.WARN,
        description="日亏损达到警告阈值",
    ),
    RiskRule(
        name="daily_loss_critical",
        level=RiskLevel.CRITICAL,
        threshold=0.05,
        action=RiskAction.FORCE_CLOSE,
        description="日亏损达到上限",
    ),
    RiskRule(
        name="max_drawdown_warning",
        level=RiskLevel.WARNING,
        threshold=0.10,
        action=RiskAction.WARN,
        description="回撤达到警告阈值",
    ),
    RiskRule(
        name="max_drawdown_critical",
        level=RiskLevel.CRITICAL,
        threshold=0.15,
        action=RiskAction.FORCE_CLOSE,
        description="回撤达到上限",
    ),
]


@dataclass
class RiskCheckResult:
    level: RiskLevel
    triggered_rules: List[RiskRule]
    recommended_action: RiskAction
    details: Dict[str, Any]


class RiskManager:
    def __init__(self, config: RiskConfig):
        self.config = config
        self.rules: List[RiskRule] = DEFAULT_RISK_RULES.copy()
        self.daily_pnl: float = 0.0
        self.peak_equity: float = 0.0
        self.current_equity: float = 0.0

    def check_risk(
        self,
        account: BaseAccount,
        positions: Dict[str, Any],
        trade_history: List[Any],
    ) -> RiskCheckResult:
        triggered: List[RiskRule] = []

        drawdown_rule = self._check_drawdown(account.balance)
        if drawdown_rule:
            triggered.append(drawdown_rule)

        if not triggered:
            return RiskCheckResult(
                level=RiskLevel.NORMAL,
                triggered_rules=[],
                recommended_action=RiskAction.NONE,
                details={},
            )

        max_level = max(r.level.value for r in triggered)
        max_action = max(r.action.value for r in triggered)
        return RiskCheckResult(
            level=RiskLevel(max_level),
            triggered_rules=triggered,
            recommended_action=RiskAction(max_action),
            details={"current_drawdown": self._calculate_drawdown()},
        )

    def _check_drawdown(self, current_balance: float) -> Optional[RiskRule]:
        self.current_equity = current_balance
        if current_balance > self.peak_equity:
            self.peak_equity = current_balance

        drawdown = self._calculate_drawdown()
        for rule in self.rules:
            if "drawdown" in rule.name and drawdown >= rule.threshold:
                return rule
        return None

    def _calculate_drawdown(self) -> float:
        if self.peak_equity <= 0:
            return 0.0
        return (self.peak_equity - self.current_equity) / self.peak_equity

    def reset_daily(self) -> None:
        self.daily_pnl = 0.0



"""临时测试：拉取现货 1m K 线"""

from __future__ import annotations

import asyncio

from datetime import datetime, timedelta, timezone

from xqtrader.data.adapters.base import MarketType
from xqtrader.data.base import DataCenterService, MarketDataRequest


async def main() -> None:
    now = datetime.now(timezone.utc)
    end_time = int(now.timestamp() * 1000)
    start_time = int((now - timedelta(minutes=15)).timestamp() * 1000)

    async with DataCenterService(market_type=MarketType.SPOT, enable_cache=False) as dc:
        data = await dc.get_market_data(
            MarketDataRequest(
                symbol="BTC/USDT",
                interval="1m",
                limit=20,
                start_time=start_time,
                end_time=end_time,
            )
        )

    ohlcv = data.get("ohlcv", {})
    closes = ohlcv.get("close", [])
    timestamps = ohlcv.get("timestamps", [])

    local_tz = timezone(timedelta(hours=8))

    print(f"现货 1m K 线数量: {len(closes)}")
    for ts, close in zip(timestamps, closes):
        ts_dt = datetime.fromtimestamp(ts / 1000, tz=timezone.utc).astimezone(local_tz)
        print(f"{ts_dt.strftime('%Y-%m-%d %H:%M')} +08:00 | close: {close}")


if __name__ == "__main__":
    asyncio.run(main())



"""技术指标计算模块

基于 talipp 库提供 60+ 种技术指标的增量计算支持。
"""

from xqtrader.indicators.incremental import (
    BaseIncrementalIndicator,
    IndicatorBar,
    IndicatorEngine,
    IndicatorRequirement,
)
from xqtrader.indicators.resampler import (
    OhlcvResampler,
    calculate_resample_ratio,
    needs_resampling,
)
from xqtrader.indicators.talipp_adapter import (
    TALIPP_REGISTRY,
    TalippIndicator,
    get_supported_indicators,
    is_indicator_supported,
)

__all__ = [
    # Core classes
    "IndicatorBar",
    "IndicatorEngine",
    "IndicatorRequirement",
    "BaseIncrementalIndicator",
    # Talipp adapter
    "TalippIndicator",
    "TALIPP_REGISTRY",
    "get_supported_indicators",
    "is_indicator_supported",
    # Resampler
    "OhlcvResampler",
    "calculate_resample_ratio",
    "needs_resampling",
]



"""基于 CCXT 的通用交易所适配器

使用 ccxt 库实现统一的交易所接口，支持 100+ 交易所。
"""

from __future__ import annotations

import asyncio
from typing import Any, Dict, Optional

from xqtrader.data.adapters.base import (
    ExchangeAdapter,
    MarketType,
    OHLCVData,
    OrderBookData,
    TickerData,
)

try:
    import ccxt.async_support as ccxt_async
    import ccxt

    CCXT_AVAILABLE = True
except ImportError:
    CCXT_AVAILABLE = False
    ccxt_async = None
    ccxt = None


class CCXTAdapter(ExchangeAdapter):
    """基于 CCXT 的通用交易所适配器

    支持所有 CCXT 支持的交易所，通过 exchange_id 指定。

    Example:
        async with CCXTAdapter("binance", MarketType.FUTURES) as adapter:
            ohlcv = await adapter.fetch_ohlcv("BTC/USDT", "1h", limit=100)
    """

    # CCXT 支持的时间间隔映射
    INTERVAL_MAP = {
        "1m": "1m",
        "3m": "3m",
        "5m": "5m",
        "15m": "15m",
        "30m": "30m",
        "1h": "1h",
        "2h": "2h",
        "4h": "4h",
        "6h": "6h",
        "8h": "8h",
        "12h": "12h",
        "1d": "1d",
        "3d": "3d",
        "1w": "1w",
        "1M": "1M",
    }

    def __init__(
        self,
        exchange_id: str = "binance",
        market_type: MarketType = MarketType.SPOT,
        api_key: Optional[str] = None,
        api_secret: Optional[str] = None,
        sandbox: bool = False,
        **kwargs: Any,
    ) -> None:
        super().__init__(exchange_id, market_type, api_key, api_secret, sandbox)

        if not CCXT_AVAILABLE:
            raise ImportError(
                "ccxt is required for CCXTAdapter. Install with: pip install ccxt"
            )

        self._exchange: Optional[ccxt_async.Exchange] = None
        self._extra_config = kwargs

    def _get_exchange_class(self) -> type:
        """获取对应的 CCXT 交易所类"""
        exchange_id = self.exchange_id.lower()

        # 处理合约市场
        if self.market_type == MarketType.FUTURES:
            # Binance USDT-M 使用 binanceusdm
            if exchange_id == "binance":
                exchange_id = "binanceusdm"
            elif exchange_id == "okx":
                exchange_id = "okx"  # OKX 统一接口
            elif exchange_id == "bybit":
                exchange_id = "bybit"
        elif self.market_type == MarketType.DELIVERY:
            if exchange_id == "binance":
                exchange_id = "binancecoinm"

        if not hasattr(ccxt_async, exchange_id):
            raise ValueError(f"Unsupported exchange: {exchange_id}")

        return getattr(ccxt_async, exchange_id)

    async def connect(self) -> None:
        """初始化 CCXT 交易所客户端"""
        exchange_class = self._get_exchange_class()

        config: Dict[str, Any] = {
            "enableRateLimit": True,  # 启用限速
            "timeout": 30000,  # 30 秒超时
        }

        if self.api_key:
            config["apiKey"] = self.api_key
        if self.api_secret:
            config["secret"] = self.api_secret
        if self.sandbox:
            config["sandbox"] = True

        # 合并额外配置
        config.update(self._extra_config)

        self._exchange = exchange_class(config)

        # 加载市场信息
        await self._exchange.load_markets()

    async def close(self) -> None:
        """关闭 CCXT 客户端"""
        if self._exchange:
            await self._exchange.close()
            self._exchange = None

    @property
    def exchange(self) -> ccxt_async.Exchange:
        """获取底层 CCXT 交易所实例"""
        if self._exchange is None:
            raise RuntimeError("Adapter not connected. Use 'async with' or call connect() first.")
        return self._exchange

    async def fetch_ohlcv(
        self,
        symbol: str,
        interval: str,
        limit: int = 100,
        start_time: Optional[int] = None,
        end_time: Optional[int] = None,
    ) -> OHLCVData:
        """获取 K 线数据"""
        normalized_symbol = self.normalize_symbol(symbol, self.market_type)
        timeframe = self.INTERVAL_MAP.get(interval, interval)

        # CCXT fetch_ohlcv 参数
        params: Dict[str, Any] = {}
        if end_time:
            params["until"] = end_time

        try:
            # CCXT 返回格式: [[timestamp, open, high, low, close, volume], ...]
            ohlcv = await self.exchange.fetch_ohlcv(
                normalized_symbol,
                timeframe=timeframe,
                since=start_time,
                limit=limit,
                params=params,
            )

            if not ohlcv:
                return OHLCVData.empty()

            return OHLCVData(
                timestamps=[int(candle[0]) for candle in ohlcv],
                open=[float(candle[1]) for candle in ohlcv],
                high=[float(candle[2]) for candle in ohlcv],
                low=[float(candle[3]) for candle in ohlcv],
                close=[float(candle[4]) for candle in ohlcv],
                volume=[float(candle[5]) for candle in ohlcv],
            )
        except ccxt.NetworkError as e:
            raise ConnectionError(f"Network error fetching OHLCV: {e}") from e
        except ccxt.ExchangeError as e:
            raise ValueError(f"Exchange error fetching OHLCV: {e}") from e

    async def fetch_ticker(self, symbol: str) -> TickerData:
        """获取最新行情"""
        normalized_symbol = self.normalize_symbol(symbol, self.market_type)

        try:
            ticker = await self.exchange.fetch_ticker(normalized_symbol)

            # CCXT 在顶层提供 markPrice 和 indexPrice（合约专用）
            mark_price = ticker.get("markPrice")
            index_price = ticker.get("indexPrice")

            return TickerData(
                symbol=normalized_symbol,
                last_price=float(ticker.get("last") or 0),
                bid_price=float(ticker.get("bid") or 0),
                ask_price=float(ticker.get("ask") or 0),
                volume_24h=float(ticker.get("quoteVolume") or ticker.get("baseVolume") or 0),
                timestamp=int(ticker.get("timestamp") or 0),
                mark_price=float(mark_price) if mark_price else None,
                index_price=float(index_price) if index_price else None,
                funding_rate=None,  # 需要单独获取
            )
        except ccxt.NetworkError as e:
            raise ConnectionError(f"Network error fetching ticker: {e}") from e
        except ccxt.ExchangeError as e:
            raise ValueError(f"Exchange error fetching ticker: {e}") from e

    async def fetch_order_book(
        self, symbol: str, limit: int = 20
    ) -> OrderBookData:
        """获取订单簿"""
        normalized_symbol = self.normalize_symbol(symbol, self.market_type)

        try:
            order_book = await self.exchange.fetch_order_book(
                normalized_symbol, limit=limit
            )

            return OrderBookData(
                symbol=normalized_symbol,
                bids=[[float(b[0]), float(b[1])] for b in order_book.get("bids", [])],
                asks=[[float(a[0]), float(a[1])] for a in order_book.get("asks", [])],
                timestamp=int(order_book.get("timestamp", 0)),
            )
        except ccxt.NetworkError as e:
            raise ConnectionError(f"Network error fetching order book: {e}") from e
        except ccxt.ExchangeError as e:
            raise ValueError(f"Exchange error fetching order book: {e}") from e

    async def fetch_ohlcv_batch(
        self,
        symbol: str,
        interval: str,
        start_time: int,
        end_time: int,
        batch_size: int = 1000,
    ) -> OHLCVData:
        """批量获取历史 K 线数据

        自动分批请求，合并结果。
        """
        all_data = OHLCVData.empty()
        current_start = start_time

        # 计算时间间隔毫秒数
        interval_ms = self._interval_to_ms(interval)

        while current_start < end_time:
            batch = await self.fetch_ohlcv(
                symbol,
                interval,
                limit=batch_size,
                start_time=current_start,
                end_time=end_time,
            )

            if len(batch) == 0:
                break

            # 合并数据
            all_data.timestamps.extend(batch.timestamps)
            all_data.open.extend(batch.open)
            all_data.high.extend(batch.high)
            all_data.low.extend(batch.low)
            all_data.close.extend(batch.close)
            all_data.volume.extend(batch.volume)

            # 更新起始时间
            last_timestamp = batch.timestamps[-1]
            current_start = last_timestamp + interval_ms

            # 避免触发限速
            await asyncio.sleep(0.1)

        return all_data

    @staticmethod
    def _interval_to_ms(interval: str) -> int:
        """将时间间隔转换为毫秒"""
        multipliers = {
            "m": 60 * 1000,
            "h": 60 * 60 * 1000,
            "d": 24 * 60 * 60 * 1000,
            "w": 7 * 24 * 60 * 60 * 1000,
            "M": 30 * 24 * 60 * 60 * 1000,
        }
        unit = interval[-1]
        value = int(interval[:-1])
        return value * multipliers.get(unit, 60 * 1000)

    def get_supported_intervals(self) -> list:
        """获取交易所支持的时间间隔"""
        if self._exchange and hasattr(self._exchange, "timeframes"):
            return list(self._exchange.timeframes.keys())
        return list(self.INTERVAL_MAP.keys())



from __future__ import annotations

import asyncio
from dataclasses import dataclass
from datetime import datetime
from typing import Any, AsyncGenerator, Dict, List, Optional

from xqtrader.accounts.futures import FuturesSimulatedAccount
from xqtrader.data.adapters.base import MarketType
from xqtrader.data.base import DataCenterService, MarketDataRequest
from xqtrader.engine.base import BaseEngine, ExecutionConfig, ExecutionEvent, ExecutionMode
from xqtrader.indicators.incremental import IndicatorBar, IndicatorEngine
from xqtrader.strategies.base import BaseStrategy, StrategyContext
from xqtrader.traders.futures import FuturesBacktestConfig, FuturesTrader, HedgePositionManager
from xqtrader.traders.events import EventsBacktestConfig, EventsTrader


@dataclass
class BacktestConfig(ExecutionConfig):
    initial_capital: float = 10000.0
    contract_type: str = "futures"  # futures | events (events 使用 UP/DOWN/HOLD)
    enable_cache: bool = True


class BacktestEngine(BaseEngine):
    def __init__(self) -> None:
        super().__init__(ExecutionMode.BACKTEST)

    @staticmethod
    def _interval_seconds(interval: str) -> int:
        normalized = interval.lower()
        if normalized.endswith("m"):
            return int(normalized[:-1]) * 60
        if normalized.endswith("h"):
            return int(normalized[:-1]) * 3600
        if normalized.endswith("d"):
            return int(normalized[:-1]) * 86400
        if normalized.endswith("w"):
            return int(normalized[:-1]) * 7 * 86400
        raise ValueError(f"unsupported interval: {interval}")

    async def run(
        self,
        strategy: BaseStrategy,
        config: ExecutionConfig,
        progress_callback: Optional[Any] = None,
    ) -> AsyncGenerator[ExecutionEvent, None]:
        if not isinstance(config, BacktestConfig):
            raise TypeError("config must be BacktestConfig")
        market_type = MarketType.FUTURES if config.contract_type == "futures" else MarketType.SPOT
        data_requirements = strategy.get_data_requirements(config.interval)
        start_time = config.start_time
        end_time = config.end_time

        request_limit = 100

        if data_requirements.use_time_range and start_time and end_time:
            interval_seconds = self._interval_seconds(config.interval)
            warmup_bars = max(data_requirements.min_bars, data_requirements.warmup_periods)

            # 计算指标预热所需的额外数据
            indicator_requirements = strategy.get_indicator_requirements()
            if indicator_requirements:
                # 找出指标中最大的 timeframe，计算其预热所需时间
                max_indicator_warmup_ms = 0
                for req in indicator_requirements.values():
                    ind_timeframe = req.get("timeframe", config.interval)
                    ind_tf_seconds = self._interval_seconds(ind_timeframe)
                    # 预热需要约 200 根目标 timeframe 的 K 线
                    ind_warmup_ms = 200 * ind_tf_seconds * 1000
                    max_indicator_warmup_ms = max(max_indicator_warmup_ms, ind_warmup_ms)

                # 使用基于指标 timeframe 的预热时间
                warmup_ms = max_indicator_warmup_ms
            else:
                warmup_ms = warmup_bars * interval_seconds * 1000

            extra_ms = data_requirements.extra_seconds * 1000
            start_time = max(0, start_time - warmup_ms - extra_ms)
            total_ms = max(0, end_time - start_time)
            required_bars = int(total_ms / (interval_seconds * 1000)) + 1
            request_limit = max(request_limit, required_bars)

        # 加载数据（MVP：占位空数据）
        async with DataCenterService(
            market_type=market_type,
            enable_cache=config.enable_cache,
        ) as data_service:
            request = MarketDataRequest(
                symbol=config.symbol,
                interval=config.interval,
                limit=request_limit,
                start_time=start_time,
                end_time=end_time,
            )
            if start_time and end_time and request_limit > 1000:
                market = await data_service.get_historical_klines_batch(request)
            else:
                market = await data_service.get_market_data(request)

        indicator_requirements = strategy.get_indicator_requirements()
        indicator_engine: Optional[IndicatorEngine] = None
        if indicator_requirements:
            indicator_engine = IndicatorEngine()
            indicator_engine.register_requirements(
                indicator_requirements,
                source_timeframe=config.interval,
            )

        # 初始化账户和交易器
        if config.contract_type == "events":
            account = FuturesSimulatedAccount(config.initial_capital)  # 事件合约也复用资金账户
            trader = EventsTrader()
            trade_config = EventsBacktestConfig(symbol=config.symbol, interval=config.interval, investment_amount=100.0)
            position_manager = None
        else:
            account = FuturesSimulatedAccount(config.initial_capital)
            trader = FuturesTrader()
            trade_config = FuturesBacktestConfig(symbol=config.symbol, interval=config.interval)
            position_manager = HedgePositionManager(config.symbol)

        ohlcv = market.get("ohlcv", {})
        closes = ohlcv.get("close", [])
        opens = ohlcv.get("open", [])
        highs = ohlcv.get("high", [])
        lows = ohlcv.get("low", [])
        volumes = ohlcv.get("volume", [])
        timestamps = ohlcv.get("timestamps", [])

        all_signals: List[Any] = []
        for idx, close in enumerate(closes):
            bar = IndicatorBar(
                timestamp=int(timestamps[idx]) if idx < len(timestamps) else 0,
                open=opens[idx] if idx < len(opens) else close,
                high=highs[idx] if idx < len(highs) else close,
                low=lows[idx] if idx < len(lows) else close,
                close=close,
                volume=volumes[idx] if idx < len(volumes) else 0.0,
                timeframe=config.interval,
            )

            incremental_indicators: Dict[str, Any] = {}
            if indicator_engine:
                indicator_engine.update(bar)
                incremental_indicators = indicator_engine.snapshot()

            yield ExecutionEvent(
                event_type="tick",
                data={
                    "bar": {
                        "timestamp": bar.timestamp,
                        "open": bar.open,
                        "high": bar.high,
                        "low": bar.low,
                        "close": bar.close,
                        "volume": bar.volume,
                        "timeframe": bar.timeframe,
                    },
                    "incremental_indicators": incremental_indicators,
                    "account_balance": account.balance,
                },
                timestamp=datetime.utcnow(),
            )

            market_slice = {
                "timestamps": timestamps[: idx + 1],
                "open": opens[: idx + 1],
                "high": highs[: idx + 1],
                "low": lows[: idx + 1],
                "close": closes[: idx + 1],
                "volume": volumes[: idx + 1],
            }

            context = StrategyContext(
                symbol=config.symbol,
                interval=config.interval,
                current_time=datetime.utcnow(),
                market_data=market_slice,
                indicators={},
                incremental_indicators=incremental_indicators,
                account_balance=account.balance,
                current_positions={},
            )

            result = strategy.execute(context)
            signals: List[Any] = result.signals if result and result.signals else []

            for signal in signals:
                all_signals.append(signal)
                if isinstance(trader, EventsTrader):
                    trade_result, record = await trader.execute_trade(signal, close, account, trade_config)  # type: ignore[arg-type]
                    yield ExecutionEvent(
                        event_type="trade",
                        data={
                            "trade_result": trade_result.__dict__,
                            "record": record,
                            "bar": {
                                "timestamp": bar.timestamp,
                                "close": bar.close,
                                "timeframe": bar.timeframe,
                            },
                        },
                        timestamp=datetime.utcnow(),
                    )
                else:
                    trade_result, records = await trader.execute_trade(
                        signal,
                        close,
                        account,
                        trade_config,  # type: ignore[arg-type]
                        position_manager,
                    )
                    yield ExecutionEvent(
                        event_type="trade",
                        data={
                            "trade_result": trade_result.__dict__,
                            "records": records,
                            "bar": {
                                "timestamp": bar.timestamp,
                                "close": bar.close,
                                "timeframe": bar.timeframe,
                            },
                        },
                        timestamp=datetime.utcnow(),
                    )

        # 完成事件
        yield ExecutionEvent(
            event_type="complete",
            data={"final_balance": account.balance, "signals": [s.__dict__ for s in all_signals]},
            timestamp=datetime.utcnow(),
        )



from __future__ import annotations

import json
import math
from dataclasses import dataclass, field
from datetime import datetime, timedelta
from pathlib import Path
from typing import Any, Dict, List, Optional


@dataclass
class EquityPoint:
    timestamp: datetime
    equity: float
    drawdown: float
    drawdown_pct: float


@dataclass
class TradeRecord:
    trade_id: str
    symbol: str
    action: str
    entry_time: datetime
    entry_price: float
    exit_time: datetime | None = None
    exit_price: float | None = None
    quantity: float = 0.0
    pnl: float = 0.0
    pnl_pct: float = 0.0
    fees: float = 0.0
    holding_period: timedelta | None = None

    def to_dict(self) -> Dict[str, Any]:
        return {
            "trade_id": self.trade_id,
            "symbol": self.symbol,
            "action": self.action,
            "entry_time": self.entry_time.isoformat(),
            "entry_price": self.entry_price,
            "exit_time": self.exit_time.isoformat() if self.exit_time else None,
            "exit_price": self.exit_price,
            "quantity": self.quantity,
            "pnl": self.pnl,
            "pnl_pct": self.pnl_pct,
            "fees": self.fees,
            "holding_period": self.holding_period.total_seconds() if self.holding_period else None,
        }


@dataclass
class BacktestReport:
    # ===== 基础信息 =====
    strategy_name: str
    symbol: str
    interval: str
    start_time: datetime
    end_time: datetime
    duration_days: int
    initial_capital: float
    final_capital: float

    # ===== 收益指标 =====
    total_return: float
    annual_return: float
    total_pnl: float

    # ===== 交易统计 =====
    total_trades: int
    winning_trades: int
    losing_trades: int
    win_rate: float
    avg_win: float
    avg_loss: float
    profit_factor: float
    avg_holding_period: timedelta

    # ===== 风险指标 =====
    max_drawdown: float
    max_drawdown_pct: float
    max_drawdown_duration: timedelta
    sharpe_ratio: float
    sortino_ratio: float
    calmar_ratio: float

    # ===== 详细数据 =====
    equity_curve: List[EquityPoint] = field(default_factory=list)
    trade_records: List[TradeRecord] = field(default_factory=list)
    monthly_returns: Dict[str, float] = field(default_factory=dict)
    daily_returns: List[float] = field(default_factory=list)

    def to_dict(self) -> Dict[str, Any]:
        return {
            "summary": {
                "strategy_name": self.strategy_name,
                "symbol": self.symbol,
                "interval": self.interval,
                "period": f"{self.start_time.isoformat()} - {self.end_time.isoformat()}",
                "duration_days": self.duration_days,
                "initial_capital": self.initial_capital,
                "final_capital": round(self.final_capital, 2),
            },
            "returns": {
                "total_return": round(self.total_return, 6),
                "annual_return": round(self.annual_return, 6),
                "total_pnl": round(self.total_pnl, 2),
            },
            "trades": {
                "total": self.total_trades,
                "winning": self.winning_trades,
                "losing": self.losing_trades,
                "win_rate": round(self.win_rate, 4),
                "avg_win": round(self.avg_win, 2),
                "avg_loss": round(self.avg_loss, 2),
                "profit_factor": round(self.profit_factor, 4),
                "avg_holding_period_seconds": self.avg_holding_period.total_seconds(),
            },
            "risk": {
                "max_drawdown": round(self.max_drawdown, 2),
                "max_drawdown_pct": round(self.max_drawdown_pct, 6),
                "max_drawdown_duration_seconds": self.max_drawdown_duration.total_seconds(),
                "sharpe_ratio": round(self.sharpe_ratio, 4),
                "sortino_ratio": round(self.sortino_ratio, 4),
                "calmar_ratio": round(self.calmar_ratio, 4),
            },
            "equity_curve": [
                {
                    "timestamp": ep.timestamp.isoformat(),
                    "equity": round(ep.equity, 2),
                    "drawdown": round(ep.drawdown, 2),
                    "drawdown_pct": round(ep.drawdown_pct, 6),
                }
                for ep in self.equity_curve
            ],
            "trade_records": [tr.to_dict() for tr in self.trade_records],
            "monthly_returns": {k: round(v, 6) for k, v in self.monthly_returns.items()},
            "daily_returns": [round(r, 6) for r in self.daily_returns],
        }


class ReportGenerator:
    TRADING_DAYS_PER_YEAR = 365
    RISK_FREE_RATE = 0.0

    @staticmethod
    def generate(
        trade_records: List[TradeRecord],
        equity_curve: List[EquityPoint],
        strategy_name: str,
        symbol: str,
        interval: str,
        initial_capital: float,
        final_capital: float,
    ) -> BacktestReport:
        if not equity_curve:
            now = datetime.now()
            return BacktestReport(
                strategy_name=strategy_name,
                symbol=symbol,
                interval=interval,
                start_time=now,
                end_time=now,
                duration_days=0,
                initial_capital=initial_capital,
                final_capital=final_capital,
                total_return=0.0,
                annual_return=0.0,
                total_pnl=0.0,
                total_trades=0,
                winning_trades=0,
                losing_trades=0,
                win_rate=0.0,
                avg_win=0.0,
                avg_loss=0.0,
                profit_factor=0.0,
                avg_holding_period=timedelta(),
                max_drawdown=0.0,
                max_drawdown_pct=0.0,
                max_drawdown_duration=timedelta(),
                sharpe_ratio=0.0,
                sortino_ratio=0.0,
                calmar_ratio=0.0,
                equity_curve=[],
                trade_records=[],
                monthly_returns={},
                daily_returns=[],
            )

        start_time = equity_curve[0].timestamp
        end_time = equity_curve[-1].timestamp
        duration = end_time - start_time
        duration_days = max(duration.days, 1)

        # ===== 收益指标 =====
        total_pnl = final_capital - initial_capital
        total_return = total_pnl / initial_capital if initial_capital > 0 else 0.0
        annual_return = ReportGenerator._annualize_return(total_return, duration_days)

        # ===== 交易统计 =====
        closed_trades = [t for t in trade_records if t.exit_time is not None]
        total_trades = len(closed_trades)
        wins = [t for t in closed_trades if t.pnl > 0]
        losses = [t for t in closed_trades if t.pnl <= 0]
        winning_trades = len(wins)
        losing_trades = len(losses)
        win_rate = winning_trades / total_trades if total_trades > 0 else 0.0

        total_win_pnl = sum(t.pnl for t in wins)
        total_loss_pnl = abs(sum(t.pnl for t in losses))
        avg_win = total_win_pnl / winning_trades if winning_trades > 0 else 0.0
        avg_loss = total_loss_pnl / losing_trades if losing_trades > 0 else 0.0
        profit_factor = total_win_pnl / total_loss_pnl if total_loss_pnl > 0 else float("inf")

        holding_periods = [t.holding_period for t in closed_trades if t.holding_period is not None]
        avg_holding_period = (
            timedelta(seconds=sum(hp.total_seconds() for hp in holding_periods) / len(holding_periods))
            if holding_periods
            else timedelta()
        )

        # ===== 风险指标 - 回撤 =====
        max_dd = 0.0
        max_dd_pct = 0.0
        max_dd_duration = timedelta()
        peak_time: Optional[datetime] = None

        for ep in equity_curve:
            if ep.drawdown > max_dd:
                max_dd = ep.drawdown
            if ep.drawdown_pct > max_dd_pct:
                max_dd_pct = ep.drawdown_pct

            if ep.drawdown == 0:
                peak_time = ep.timestamp
            elif peak_time is not None:
                dd_dur = ep.timestamp - peak_time
                if dd_dur > max_dd_duration:
                    max_dd_duration = dd_dur

        # ===== 日收益率 =====
        daily_returns = ReportGenerator._compute_daily_returns(equity_curve)

        # ===== 月度收益 =====
        monthly_returns = ReportGenerator._compute_monthly_returns(equity_curve)

        # ===== 风险调整收益比率 =====
        sharpe_ratio = ReportGenerator._compute_sharpe(daily_returns)
        sortino_ratio = ReportGenerator._compute_sortino(daily_returns)
        calmar_ratio = annual_return / max_dd_pct if max_dd_pct > 0 else float("inf")

        return BacktestReport(
            strategy_name=strategy_name,
            symbol=symbol,
            interval=interval,
            start_time=start_time,
            end_time=end_time,
            duration_days=duration_days,
            initial_capital=initial_capital,
            final_capital=final_capital,
            total_return=total_return,
            annual_return=annual_return,
            total_pnl=total_pnl,
            total_trades=total_trades,
            winning_trades=winning_trades,
            losing_trades=losing_trades,
            win_rate=win_rate,
            avg_win=avg_win,
            avg_loss=avg_loss,
            profit_factor=profit_factor,
            avg_holding_period=avg_holding_period,
            max_drawdown=max_dd,
            max_drawdown_pct=max_dd_pct,
            max_drawdown_duration=max_dd_duration,
            sharpe_ratio=sharpe_ratio,
            sortino_ratio=sortino_ratio,
            calmar_ratio=calmar_ratio,
            equity_curve=equity_curve,
            trade_records=trade_records,
            monthly_returns=monthly_returns,
            daily_returns=daily_returns,
        )

    @staticmethod
    def _json_default(obj: Any) -> Any:
        if isinstance(obj, float):
            if math.isinf(obj):
                return "Infinity" if obj > 0 else "-Infinity"
            if math.isnan(obj):
                return "NaN"
        raise TypeError(f"Object of type {type(obj)} is not JSON serializable")

    @staticmethod
    def export(
        report: BacktestReport,
        format: str = "json",
        path: Optional[str] = None,
    ) -> str:
        if format == "json":
            def _sanitize(obj: Any) -> Any:
                if isinstance(obj, dict):
                    return {k: _sanitize(v) for k, v in obj.items()}
                if isinstance(obj, list):
                    return [_sanitize(v) for v in obj]
                if isinstance(obj, float) and (math.isinf(obj) or math.isnan(obj)):
                    return None
                return obj

            content = json.dumps(_sanitize(report.to_dict()), indent=2, ensure_ascii=False)
        else:
            raise ValueError(f"Unsupported format: {format}. Supported: json")

        if path is not None:
            Path(path).parent.mkdir(parents=True, exist_ok=True)
            Path(path).write_text(content, encoding="utf-8")

        return content

    # ===== 内部计算方法 =====

    @staticmethod
    def _annualize_return(total_return: float, duration_days: int) -> float:
        if duration_days <= 0:
            return 0.0
        years = duration_days / ReportGenerator.TRADING_DAYS_PER_YEAR
        if years <= 0:
            return 0.0
        return (1 + total_return) ** (1 / years) - 1

    @staticmethod
    def _compute_daily_returns(equity_curve: List[EquityPoint]) -> List[float]:
        if len(equity_curve) < 2:
            return []

        daily_equities: Dict[str, float] = {}
        for ep in equity_curve:
            day_key = ep.timestamp.strftime("%Y-%m-%d")
            daily_equities[day_key] = ep.equity

        days = sorted(daily_equities.keys())
        if len(days) < 2:
            return []

        returns = []
        for i in range(1, len(days)):
            prev_eq = daily_equities[days[i - 1]]
            curr_eq = daily_equities[days[i]]
            r = (curr_eq - prev_eq) / prev_eq if prev_eq > 0 else 0.0
            returns.append(r)
        return returns

    @staticmethod
    def _compute_monthly_returns(equity_curve: List[EquityPoint]) -> Dict[str, float]:
        if len(equity_curve) < 2:
            return {}

        monthly_equities: Dict[str, float] = {}
        first_of_month: Dict[str, float] = {}

        for ep in equity_curve:
            month_key = ep.timestamp.strftime("%Y-%m")
            monthly_equities[month_key] = ep.equity
            if month_key not in first_of_month:
                first_of_month[month_key] = ep.equity

        months = sorted(monthly_equities.keys())
        result: Dict[str, float] = {}
        for i, month in enumerate(months):
            start_eq = first_of_month[month]
            if i > 0:
                start_eq = monthly_equities[months[i - 1]]
            end_eq = monthly_equities[month]
            result[month] = (end_eq - start_eq) / start_eq if start_eq > 0 else 0.0
        return result

    @staticmethod
    def _compute_sharpe(daily_returns: List[float], risk_free_rate: float = 0.0) -> float:
        if len(daily_returns) < 2:
            return 0.0
        daily_rf = risk_free_rate / ReportGenerator.TRADING_DAYS_PER_YEAR
        excess = [r - daily_rf for r in daily_returns]
        mean = sum(excess) / len(excess)
        variance = sum((r - mean) ** 2 for r in excess) / (len(excess) - 1)
        std = math.sqrt(variance) if variance > 0 else 0.0
        if std == 0:
            return 0.0
        return (mean / std) * math.sqrt(ReportGenerator.TRADING_DAYS_PER_YEAR)

    @staticmethod
    def _compute_sortino(daily_returns: List[float], risk_free_rate: float = 0.0) -> float:
        if len(daily_returns) < 2:
            return 0.0
        daily_rf = risk_free_rate / ReportGenerator.TRADING_DAYS_PER_YEAR
        excess = [r - daily_rf for r in daily_returns]
        mean = sum(excess) / len(excess)
        downside = [r for r in excess if r < 0]
        if not downside:
            return float("inf") if mean > 0 else 0.0
        downside_variance = sum(r**2 for r in downside) / (len(excess) - 1)
        downside_std = math.sqrt(downside_variance) if downside_variance > 0 else 0.0
        if downside_std == 0:
            return 0.0
        return (mean / downside_std) * math.sqrt(ReportGenerator.TRADING_DAYS_PER_YEAR)


class ReportCollector:
    """从引擎事件流中收集数据，自动构建报告。

    用法:
        collector = ReportCollector(strategy_name, symbol, interval, initial_capital)
        async for event in engine.run(strategy, config):
            collector.collect(event)
            # ... 自己的业务逻辑 ...
        report = collector.build()
        ReportGenerator.export(report, path="report.json")
    """

    def __init__(
        self,
        strategy_name: str,
        symbol: str,
        interval: str,
        initial_capital: float,
        start_time: Optional[int] = None,
        end_time: Optional[int] = None,
    ) -> None:
        self.strategy_name = strategy_name
        self.symbol = symbol
        self.interval = interval
        self.initial_capital = initial_capital
        self.final_capital = initial_capital
        self._start_ms = start_time
        self._end_ms = end_time

        self._equity_curve: List[EquityPoint] = []
        self._peak_equity: float = initial_capital
        self._open_trades: Dict[str, Dict[str, Any]] = {}  # side -> open trade info
        self._trade_records: List[TradeRecord] = []
        self._trade_counter: int = 0

    def collect(self, event: Any) -> None:
        event_type = event.event_type
        data = event.data

        if event_type == "tick":
            self._collect_tick(data)
        elif event_type == "trade":
            self._collect_trade(data)
        elif event_type == "complete":
            self.final_capital = data.get("final_balance", self.final_capital)

    def build(self) -> BacktestReport:
        return ReportGenerator.generate(
            trade_records=self._trade_records,
            equity_curve=self._equity_curve,
            strategy_name=self.strategy_name,
            symbol=self.symbol,
            interval=self.interval,
            initial_capital=self.initial_capital,
            final_capital=self.final_capital,
        )

    def _in_window(self, ts: int) -> bool:
        if self._start_ms is not None and ts < self._start_ms:
            return False
        if self._end_ms is not None and ts > self._end_ms:
            return False
        return True

    def _collect_tick(self, data: Dict[str, Any]) -> None:
        balance = data.get("account_balance")
        bar = data.get("bar", {})
        ts = bar.get("timestamp", 0)
        if balance is None or ts == 0:
            return
        if not self._in_window(ts):
            return

        equity = float(balance)
        self._peak_equity = max(self._peak_equity, equity)
        drawdown = self._peak_equity - equity
        drawdown_pct = drawdown / self._peak_equity if self._peak_equity > 0 else 0.0

        self._equity_curve.append(
            EquityPoint(
                timestamp=datetime.utcfromtimestamp(ts / 1000) if ts > 1e9 else datetime.utcfromtimestamp(ts),
                equity=equity,
                drawdown=drawdown,
                drawdown_pct=drawdown_pct,
            )
        )

    def _collect_trade(self, data: Dict[str, Any]) -> None:
        bar = data.get("bar", {})
        ts = bar.get("timestamp", 0)
        if not self._in_window(ts):
            return
        trade_time = datetime.utcfromtimestamp(ts / 1000) if ts > 1e9 else datetime.utcfromtimestamp(ts)
        price = bar.get("close", 0.0)
        trade_result = data.get("trade_result", {})

        # 永续合约: records 是列表
        records = data.get("records")
        if records and isinstance(records, list):
            for rec in records:
                self._process_futures_record(rec, trade_time, price, trade_result)
            return

        # 事件合约: record 是单个字典
        record = data.get("record")
        if record and isinstance(record, dict):
            self._process_events_record(record, trade_time, price, trade_result)

    def _process_futures_record(
        self, rec: Dict[str, Any], trade_time: datetime, price: float, trade_result: Dict[str, Any]
    ) -> None:
        action = rec.get("action", "")
        side = rec.get("side", "")
        size = rec.get("size", 0.0)
        trade_price = rec.get("price", price)

        action_upper = action.upper()
        is_open = action_upper in ("LONG", "SHORT", "BUY", "SELL", "OPEN_LONG", "OPEN_SHORT")
        is_close = action_upper.startswith("CLOSE")

        if is_open:
            self._open_trades[side] = {
                "entry_time": trade_time,
                "entry_price": trade_price,
                "size": size,
                "action": action,
                "side": side,
            }
        elif is_close:
            close_side = side
            if not close_side:
                if "LONG" in action_upper:
                    close_side = "long"
                elif "SHORT" in action_upper:
                    close_side = "short"

            open_trade = self._open_trades.pop(close_side, None)
            if open_trade:
                self._trade_counter += 1
                pnl = rec.get("pnl", trade_result.get("pnl", 0.0))
                fees = rec.get("fees", trade_result.get("fees", 0.0))
                entry_price = open_trade["entry_price"]
                nominal = entry_price * open_trade["size"]

                self._trade_records.append(
                    TradeRecord(
                        trade_id=f"T{self._trade_counter:04d}",
                        symbol=self.symbol,
                        action=open_trade["action"],
                        entry_time=open_trade["entry_time"],
                        entry_price=entry_price,
                        exit_time=trade_time,
                        exit_price=trade_price,
                        quantity=open_trade["size"],
                        pnl=pnl,
                        pnl_pct=pnl / nominal if nominal > 0 else 0.0,
                        fees=fees,
                        holding_period=trade_time - open_trade["entry_time"],
                    )
                )

    def _process_events_record(
        self, rec: Dict[str, Any], trade_time: datetime, price: float, trade_result: Dict[str, Any]
    ) -> None:
        if rec.get("skipped"):
            return

        self._trade_counter += 1
        pnl = trade_result.get("pnl", 0.0)
        fees = trade_result.get("fees", 0.0)

        self._trade_records.append(
            TradeRecord(
                trade_id=f"T{self._trade_counter:04d}",
                symbol=self.symbol,
                action=rec.get("action", ""),
                entry_time=trade_time,
                entry_price=price,
                exit_time=trade_time,
                exit_price=price,
                quantity=1.0,
                pnl=pnl,
                pnl_pct=pnl / price if price > 0 else 0.0,
                fees=fees,
                holding_period=timedelta(),
            )
        )



"""MACD 事件合约回测示例"""

from __future__ import annotations

import asyncio

from datetime import datetime, timedelta, timezone

from xqtrader.engine.backtest import BacktestConfig, BacktestEngine
from xqtrader.reports.base import ReportCollector, ReportGenerator
from macd_strategy import MACDConfig, MACDStrategy


async def main() -> None:
    macd_config = MACDConfig(
        timeframe="5m",
        require_histogram_confirm=False,
    )
    strategy = MACDStrategy(config=macd_config)

    engine = BacktestEngine()
    now = datetime.now(timezone.utc)
    last_closed = now.replace(minute=0, second=0, microsecond=0)
    if last_closed > now:
        last_closed -= timedelta(hours=1)
    end_time = int(last_closed.timestamp() * 1000)
    start_time = int((last_closed - timedelta(hours=3)).timestamp() * 1000)

    requested_start_time = start_time
    requested_end_time = end_time

    config = BacktestConfig(
        symbol="BTC/USDT",
        interval="1m",
        initial_capital=10000.0,
        start_time=start_time,
        end_time=end_time,
        contract_type="events",
        enable_cache=False,
    )

    collector = ReportCollector(
        strategy_name=strategy.name,
        symbol=config.symbol,
        interval=config.interval,
        initial_capital=config.initial_capital,
        start_time=requested_start_time,
        end_time=requested_end_time,
    )

    indicator_id = f"macd_{macd_config.timeframe}"
    warmed_up = False

    local_tz = datetime.now().astimezone().tzinfo

    signal_count = 0
    latest_macd = {}

    async for event in engine.run(strategy, config):
        collector.collect(event)
        if event.event_type == "tick":
            indicators = event.data.get("incremental_indicators", {})
            timeframe_state = indicators.get("by_timeframe", {}).get(macd_config.timeframe, {})
            is_warmed_up = timeframe_state.get("is_warmed_up", indicators.get("is_warmed_up", False))
            warmed_up = is_warmed_up

            macd_data = indicators.get("macd", {}).get(indicator_id, {})
            if macd_data:
                latest_macd = macd_data

        elif event.event_type == "trade":
            trade_bar = event.data.get("bar", {})
            trade_ts = trade_bar.get("timestamp")
            record = event.data.get("record")

            if not record:
                continue

            if record.get("skipped") is True:
                continue

            if (
                not warmed_up
                or (isinstance(trade_ts, int) and trade_ts > 0 and trade_ts < requested_start_time)
                or (isinstance(trade_ts, int) and trade_ts > 0 and trade_ts > requested_end_time)
            ):
                continue

            signal_count += 1
            trade_result = event.data.get("trade_result", {})
            trade_time = datetime.fromtimestamp(trade_ts / 1000).astimezone(local_tz)

            action = record.get("action", "N/A")
            pnl = trade_result.get("pnl", 0.0)
            win = record.get("win")

            macd_val = latest_macd.get("macd")
            signal_val = latest_macd.get("signal_line")
            hist_val = latest_macd.get("histogram")

            macd_str = f"{macd_val:.4f}" if isinstance(macd_val, (int, float)) else "N/A"
            signal_str = f"{signal_val:.4f}" if isinstance(signal_val, (int, float)) else "N/A"
            hist_str = f"{hist_val:.4f}" if isinstance(hist_val, (int, float)) else "N/A"
            win_str = "赢" if win is True else "输" if win is False else "N/A"

            print(
                f"\nSignal #{signal_count} | {trade_time.strftime('%Y-%m-%d %H:%M')}\n"
                f"   Action: {action}\n"
                f"   Price: {trade_bar.get('close')}\n"
                f"   Result: {win_str} | PnL: {pnl:.2f}\n"
                f"   MACD: {macd_str} | Signal: {signal_str} | Hist: {hist_str}"
            )

        elif event.event_type == "complete":
            print(f"\n{'='*50}")
            print(f"回测完成 | 总信号数: {signal_count} | 最终余额: {event.data.get('final_balance')}")

    # 生成报告
    report = collector.build()
    report_path = "report_events.json"
    ReportGenerator.export(report, format="json", path=report_path)
    print(f"\n报告已导出: {report_path}")

    summary = report.to_dict()
    print(f"\n===== 回测报告摘要 =====")
    print(f"策略: {summary['summary']['strategy_name']}")
    print(f"标的: {summary['summary']['symbol']}")
    print(f"区间: {summary['summary']['period']}")
    print(f"总收益率: {summary['returns']['total_return']:.2%}")
    print(f"总盈亏: {summary['returns']['total_pnl']:.2f}")
    print(f"总交易: {summary['trades']['total']}")
    print(f"胜率: {summary['trades']['win_rate']:.2%}")
    print(f"盈亏比: {summary['trades']['profit_factor']:.2f}")
    print(f"最大回撤: {summary['risk']['max_drawdown_pct']:.2%}")
    print(f"夏普比率: {summary['risk']['sharpe_ratio']:.2f}")


if __name__ == "__main__":
    asyncio.run(main())
